# -*- coding: utf-8 -*-
import logging

from pprint import pprint as pp

import click

from scout.utils.requests import fetch_mim_files
from scout.utils.date import get_date

from scout.load.panel import (load_panel_app, load_panel)

from scout.exceptions import IntegrityError

LOG = logging.getLogger(__name__)


@click.command('panel', short_help='Load a gene panel')
@click.argument('path', 
    type=click.Path(exists=True), 
    required=False
)
@click.option('--panel-id', 
    help="The panel identifier name",
)
@click.option('--institute',
    help="Specify the owner of the panel. Defaults to cust000."
)
@click.option('-d', '--date', 
    help='date of gene panel on format 2017-12-24, default is today.'
)
@click.option('-n', '--display-name', 
    help='display name for the panel, optional'
)
@click.option('-v', '--version',type=float,)
@click.option('-t', '--panel-type', 
    default='clinical', 
    show_default=True,
    type=click.Choice(['clinical', 'research'])
)
@click.option('--omim',
    is_flag=True,
    help="Load the OMIM-AUTO panel into scout. A OMIM api key is required to do this(https://omim.org/api)."
)
@click.option('--api-key',
    help="A OMIM api key, see https://omim.org/api."
)
@click.option('--panel-app',
    is_flag=True,
    help="Load all PanelApp panels into scout."
)
@click.pass_context
def panel(context, path, date, display_name, version, panel_type, panel_id, institute, omim, api_key, panel_app):
    """Add a gene panel to the database."""

    adapter = context.obj['adapter']
    institute = institute or 'cust000'

    if omim:
        api_key = api_key or context.obj.get('omim_api_key')
        if not api_key:
            LOG.warning("Please provide a omim api key to load the omim gene panel")
            context.abort()
        #Check if OMIM-AUTO exists
        if adapter.gene_panel(panel_id='OMIM-AUTO'):
            LOG.warning("OMIM-AUTO already exists in database")
            LOG.info("To create a new version use scout update omim")
            return

        # Here we know that there is no panel loaded
        try:
            adapter.load_omim_panel(api_key, institute=institute)
        except Exception as err:
            LOG.error(err)
            context.abort()
    
    if panel_app:
        # try:
        load_panel_app(adapter, panel_id, institute=institute)
        # except Exception as err:
        #     LOG.warning(err)
        #     context.abort()

    if (omim or panel_app):
        return

    if path is None:
        LOG.info("Please provide a panel")
        return
    try:
        load_panel(path, adapter, date, display_name, version, panel_type, panel_id, institute)
    except Exception as err:
        LOG.warning(err)
        context.abort()




import logging
from pprint import pprint as pp

logger = logging.getLogger(__name__)


def parse_hgnc_line(line, header):
    """Parse an hgnc formated line

        Args:
            line(list): A list with hgnc gene info
            header(list): A list with the header info

        Returns:
            hgnc_info(dict): A dictionary with the relevant info
    """
    hgnc_gene = {}
    line = line.rstrip().split('\t')
    raw_info = dict(zip(header, line))
    # Skip all genes that have status withdrawn
    if 'Withdrawn' in raw_info['status']:
        return hgnc_gene
    
    hgnc_symbol = raw_info['symbol']
    hgnc_gene['hgnc_symbol'] = hgnc_symbol
    hgnc_gene['hgnc_id'] = int(raw_info['hgnc_id'].split(':')[-1])
    hgnc_gene['description'] = raw_info['name']
    
    # We want to have the current symbol as an alias
    aliases = set([hgnc_symbol, hgnc_symbol.upper()])
    # We then need to add both the previous symbols and
    # alias symbols
    previous_names = raw_info['prev_symbol']
    if previous_names:
        for alias in previous_names.strip('"').split('|'):
            aliases.add(alias)

    alias_symbols = raw_info['alias_symbol']
    if alias_symbols:
        for alias in alias_symbols.strip('"').split('|'):
            aliases.add(alias)

    hgnc_gene['previous_symbols'] = list(aliases)

    # We need the ensembl_gene_id to link the genes with ensembl
    hgnc_gene['ensembl_gene_id'] = raw_info.get('ensembl_gene_id')

    omim_id = raw_info.get('omim_id')
    if omim_id:
        hgnc_gene['omim_id'] = int(omim_id.strip('"').split('|')[0])
    else:
        hgnc_gene['omim_id'] = None

    entrez_id = hgnc_gene['entrez_id'] = raw_info.get('entrez_id')
    if entrez_id:
        hgnc_gene['entrez_id'] = int(entrez_id)
    else:
        hgnc_gene['entrez_id'] = None

    # These are the primary transcripts according to HGNC
    ref_seq = raw_info.get('refseq_accession')
    if ref_seq:
        hgnc_gene['ref_seq'] = ref_seq.strip('"').split('|')
    else:
        hgnc_gene['ref_seq'] = []

    uniprot_ids = raw_info.get('uniprot_ids')
    if uniprot_ids:
        hgnc_gene['uniprot_ids'] = uniprot_ids.strip('""').split('|')
    else:
        hgnc_gene['uniprot_ids'] = []

    ucsc_id = raw_info.get('ucsc_id')
    if ucsc_id:
        hgnc_gene['ucsc_id'] = ucsc_id
    else:
        hgnc_gene['ucsc_id'] = None

    vega_id = raw_info.get('vega_id')
    if vega_id:
        hgnc_gene['vega_id'] = vega_id
    else:
        hgnc_gene['vega_id'] = None

    return hgnc_gene


def parse_hgnc_genes(lines):
    """Parse lines with hgnc formated genes

        This is designed to take a dump with genes from HGNC.
        This is downloaded from:
        ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/hgnc_complete_set.txt

        Args:
            lines(iterable(str)): An iterable with HGNC formated genes
        Yields:
            hgnc_gene(dict): A dictionary with the relevant information
    """
    header = []
    logger.info("Parsing hgnc genes...")
    for index, line in enumerate(lines):
        if index == 0:
            header = line.split('\t')
        elif len(line) > 1:
            hgnc_gene = parse_hgnc_line(line=line, header=header)
            if hgnc_gene:
                yield hgnc_gene



import logging
import click

LOG = logging.getLogger(__name__)


@click.command('cases', short_help='Display cases')
@click.option('-i', '--institute',
    help="What institute to show cases from"
)
@click.option('-d', '--display-name',
    help="Search with display name"
)
@click.option('-c', '--case-id',
    help="Search for a specific case"
)
@click.option('--nr-variants',
    is_flag=True,
    help="Show number of clinical and research variants"
)
@click.option('--variants-treshold',
    default=0,
    help="Only show cases with more variants than treshold"
)
@click.pass_context
def cases(context, institute, display_name, case_id, nr_variants, variants_treshold):
    """Display cases from the database"""
    LOG.info("Running scout view institutes")
    adapter = context.obj['adapter']

    models = []
    if case_id:
        case_obj = adapter.case(case_id=case_id)
        if case_obj:
            models.append(case_obj)

    else:
        models = adapter.cases(collaborator=institute, name_query=display_name)
        models = [case_obj for case_obj in models]

    if not models:
        LOG.info("No cases could be found")
        return
    
    header = ['case_id', 'display_name', 'institute']
    
    if variants_treshold:
        LOG.info("Only show cases with more than %s variants", variants_treshold)
        nr_variants = True
    
    if nr_variants:
        LOG.info("Displaying number of variants for each case")
        header.append('clinical')
        header.append('research')
    
    
    click.echo("#"+'\t'.join(header))
    for model in models:
        output_str = "{:<12}\t{:<12}\t{:<12}"
        output_values = [model['_id'],model['display_name'],model['owner']]
        if nr_variants:
            output_str += "\t{:<12}\t{:<12}"
            nr_clinical = 0
            nr_research = 0
            variants = adapter.variant_collection.find({'case_id':model['_id']})
            i = 0
            for i, var in enumerate(variants, 1):
                if var['variant_type'] == 'clinical':
                    nr_clinical += 1
                else:
                    nr_research += 1
            output_values.extend([nr_clinical, nr_research])
            
            if variants_treshold and i < variants_treshold:
                LOG.debug("Case %s had to few variants, skipping", model['_id'])
                continue
            
        click.echo(output_str.format(*output_values))



from scout.build.user import build_user


def test_get_insert_user(adapter):
    user_info = {
        'email': 'clark.kent@mail.com',
        'location': 'here',
        'name': 'Clark Kent',
        'institutes': ['test-1'],
        
    }
    user_obj = build_user(user_info)
    ## GIVEN a empty adapter
    
    assert adapter.user(email=user_info['email']) is None

    ## WHEN insaerting a user
    user_obj = adapter.add_user(user_obj)

    ## THEN assert that the user is in the database
    
    assert user_obj['_id'] == user_info['email']


def test_get_nonexisting_user(adapter):
    """docstring for test_get_nonexisting_user"""
    user_obj = adapter.user(email='john.doe@mail.com')
    assert user_obj == None
    


# -*- coding: utf-8 -*-
import logging

from scout.adapter import MongoAdapter
from scout.exceptions import IntegrityError, DataNotFoundError

logger = logging.getLogger(__name__)


def load_delivery_report(adapter: MongoAdapter,
                         report_path: str,
                         case_id: str,
                         update: bool = False):
    """ Load a delivery report into a case in the database

    If the report already exists the function will exit.
    If the user want to load a report that is already in the database
    'update' has to be 'True'

    Args:
        adapter     (MongoAdapter): Connection to the database
        report_path (string):       Path to delivery report
        case_id     (string):       Optional case identifier
        update      (bool):         If an existing report should be replaced
        
    Returns:
        updated_case(dict)

    """

    case_obj = adapter.case(
        case_id=case_id,
    )

    if case_obj is None:
        raise DataNotFoundError("no case found")

    if not case_obj.get('delivery_report'):
        _put_report_in_case_root(case_obj, report_path)
    else:
        if update:
            _put_report_in_case_root(case_obj, report_path)
        else:
            raise IntegrityError('Existing delivery report found, use update = True to '
                                 'overwrite')

    logger.info('Saving report for case {} in database'.format(case_obj['_id']))
    return adapter.replace_case(case_obj)


def _put_report_in_case_root(case_obj, report_path):
    case_obj['delivery_report'] = report_path
    return True



# These are the valid SO terms with corresponfing severity rank
SO_TERMS = {
    'transcript_ablation': {'rank':1, 'region':'exonic'},
    'splice_donor_variant': {'rank':2, 'region':'splicing'},
    'splice_acceptor_variant': {'rank':3, 'region':'splicing'},
    'stop_gained': {'rank':4, 'region':'exonic'},
    'frameshift_variant': {'rank':5, 'region':'exonic'},
    'stop_lost': {'rank':6, 'region':'exonic'},
    'start_lost': {'rank': 7, 'region': 'exonic'},
    'initiator_codon_variant': {'rank':8, 'region':'exonic'},
    'inframe_insertion': {'rank':9, 'region':'exonic'},
    'inframe_deletion': {'rank':10, 'region':'exonic'},
    'missense_variant': {'rank':11, 'region':'exonic'},
    'protein_altering_variant': {'rank':12, 'region':'exonic'},
    'transcript_amplification': {'rank':13, 'region':'exonic'},
    'splice_region_variant': {'rank':14, 'region':'splicing'},
    'incomplete_terminal_codon_variant': {'rank':15, 'region':'exonic'},
    'synonymous_variant': {'rank':16, 'region':'exonic'},
    'start_retained_variant': {'rank':17, 'region':'exonic'},
    'stop_retained_variant': {'rank':18, 'region':'exonic'},
    'coding_sequence_variant': {'rank':19, 'region':'exonic'},
    'mature_miRNA_variant': {'rank':20, 'region':'ncRNA_exonic'},
    '5_prime_UTR_variant': {'rank':21, 'region':'5UTR'},
    '3_prime_UTR_variant': {'rank':22, 'region':'3UTR'},
    'non_coding_transcript_exon_variant': {'rank':23, 'region':'ncRNA_exonic'},
    'non_coding_exon_variant': {'rank':24, 'region':'ncRNA_exonic'},
    'non_coding_transcript_variant': {'rank':25, 'region':'ncRNA_exonic'},
    'nc_transcript_variant': {'rank':26, 'region':'ncRNA_exonic'},
    'intron_variant': {'rank':27, 'region':'intronic'},
    'NMD_transcript_variant': {'rank':28, 'region':'ncRNA'},
    'upstream_gene_variant': {'rank':29, 'region':'upstream'},
    'downstream_gene_variant': {'rank':30, 'region':'downstream'},
    'TFBS_ablation': {'rank':31, 'region':'TFBS'},
    'TFBS_amplification': {'rank':32, 'region':'TFBS'},
    'TF_binding_site_variant': {'rank':33, 'region':'TFBS'},
    'regulatory_region_ablation': {'rank':34, 'region':'regulatory_region'},
    'regulatory_region_amplification': {'rank':35, 'region':'regulatory_region'},
    'regulatory_region_variant': {'rank':36, 'region':'regulatory_region'},
    'feature_elongation': {'rank':37, 'region':'genomic_feature'},
    'feature_truncation': {'rank':38, 'region':'genomic_feature'},
    'intergenic_variant': {'rank':39, 'region':'intergenic_variant'}
}

SO_TERM_KEYS = list(SO_TERMS.keys())

SEVERE_SO_TERMS = (
    'transcript_ablation',
    'splice_donor_variant',
    'splice_acceptor_variant',
    'stop_gained',
    'frameshift_variant',
    'stop_lost',
    'start_lost',
    'initiator_codon_variant',
    'inframe_insertion',
    'inframe_deletion',
    'missense_variant',
    'protein_altering_variant',
    'transcript_amplification',
    'splice_region_variant',
    'incomplete_terminal_codon_variant',
    'synonymous_variant',
    'start_retained_variant',
    'stop_retained_variant',
    'coding_sequence_variant'
)



"""scout.load.panel.py

functions to load panels into the database

"""

import logging
import json

from pprint import pprint as pp

from scout.utils.handle import get_file_handle
from scout.parse.panel import (get_panel_info, parse_panel_app_panel, parse_gene_panel)
from scout.utils.requests import get_request

LOG = logging.getLogger(__name__)


def load_panel(panel_path, adapter, date=None, display_name=None, version=None, panel_type=None, 
               panel_id=None, institute=None):
    """Load a manually curated gene panel into scout
    
    Args:
        panel_path(str): path to gene panel file
        adapter(scout.adapter.MongoAdapter)
        date(str): date of gene panel on format 2017-12-24
        display_name(str)
        version(float)
        panel_type(str)
        panel_id(str)
        institute(str)
    
    """
    panel_lines = get_file_handle(panel_path)

    try:
        # This will parse panel metadata if includeed in panel file
        panel_info = get_panel_info(
            panel_lines=panel_lines,
            panel_id=panel_id,
            institute=institute,
            version=version,
            date=date,
            display_name=display_name
            )
    except Exception as err:
        raise err

    version = None
    if panel_info.get('version'):
        version = float(panel_info['version'])

    panel_id = panel_info['panel_id']
    display_name = panel_info['display_name'] or panel_id
    institute = panel_info['institute']
    date = panel_info['date']

    if not institute:
        raise SyntaxError("A Panel has to belong to a institute")

    #Check if institute exists in database
    if not adapter.institute(institute):
        raise SyntaxError("Institute {0} does not exist in database".format(institute))

    if not panel_id:
        raise SyntaxError("A Panel has to have a panel id")
    
    if version:
        existing_panel = adapter.gene_panel(panel_id, version)
    else:
        ## Assuming version 1.0
        existing_panel = adapter.gene_panel(panel_id)
        version = 1.0
        LOG.info("Set version to %s", version)

    if existing_panel:
        LOG.info("found existing panel")
        if version == existing_panel['version']:
            LOG.warning("Panel with same version exists in database")
            LOG.info("Reload with updated version")
            raise SyntaxError()
        display_name = display_name or existing_panel['display_name']
        institute = institute or existing_panel['institute']
    
    parsed_panel = parse_gene_panel(
        path=panel_path,
        institute=institute,
        panel_type=panel_type,
        date=date,
        version=version,
        panel_id=panel_id,
        display_name=display_name,
    )
    
    try:
        adapter.load_panel(parsed_panel=parsed_panel)
    except Exception as err:
        raise err

def load_panel_app(adapter, panel_id=None, institute='cust000'):
    """Load PanelApp panels into scout database
    
    If no panel_id load all PanelApp panels 
    
    Args:
        adapter(scout.adapter.MongoAdapter)
        panel_id(str): The panel app panel id
    """
    base_url = 'https://panelapp.genomicsengland.co.uk/WebServices/{0}/'
    
    hgnc_map = adapter.genes_by_alias()
    
    if panel_id:
        panel_ids = [panel_id]

    if not panel_id:
        
        LOG.info("Fetching all panel app panels")
        data = get_request(base_url.format('list_panels'))
    
        json_lines = json.loads(data)
        
        panel_ids = [panel_info['Panel_Id'] for panel_info in json_lines['result']]
    
    for panel_id in panel_ids:
        panel_data = get_request(base_url.format('get_panel') + panel_id)
        
        parsed_panel = parse_panel_app_panel(
            panel_info = json.loads(panel_data)['result'], 
            hgnc_map=hgnc_map,
            institute=institute
        )
        parsed_panel['panel_id'] = panel_id
        
        if len(parsed_panel['genes']) == 0:
            LOG.warning("Panel {} is missing genes. Skipping.".format(parsed_panel['display_name']))
            continue
        
        try:
            adapter.load_panel(parsed_panel=parsed_panel)
        except Exception as err:
            raise err

        
        
    
    
    


import pkg_resources


###### Files ######

# Cytoband

cytobands_file = 'resources/cytoBand.txt.gz'

###### Paths ######

# Cytoband path

cytobands_path = pkg_resources.resource_filename('scout', cytobands_file)



import pytest
from pprint import pprint as pp

from scout.exceptions.database import IntegrityError
from scout.server.blueprints.variants.controllers import variants

from cyvcf2 import VCF

def test_load_variant(real_populated_database, variant_obj):
    """Test to load a variant into a real mongo database"""
    adapter = real_populated_database
    # GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0

    # WHEN loading a variant into the database
    adapter.load_variant(variant_obj=variant_obj)
    # THEN assert the variant is loaded

    assert adapter.variant_collection.find_one()

def test_load_variant_twice(real_populated_database, variant_obj):
    """Test to load a variant into a mongo database"""
    adapter = real_populated_database
    # GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0

    # WHEN loading a variant into the database twice
    adapter.load_variant(variant_obj=variant_obj)

    # THEN a IntegrityError should be raised
    with pytest.raises(IntegrityError):
        adapter.load_variant(variant_obj=variant_obj)

def test_load_variants(real_populated_database, case_obj, variant_clinical_file):
    """Test to load a variant into a mongo database"""
    adapter = real_populated_database
    # GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0

    # WHEN loading a variant into the database
    rank_threshold = 0
    adapter.load_variants(
            case_obj=case_obj,
            variant_type='clinical',
            category='snv',
            rank_threshold=rank_threshold,
            chrom=None,
            start=None,
            end=None,
    )
    # THEN assert the variant is loaded

    assert adapter.variant_collection.find().count() > 0

    for variant in adapter.variant_collection.find():
        if variant['chromosome'] != 'MT':
            assert variant['rank_score'] >= rank_threshold
        assert variant['category'] == 'snv'
        assert variant['variant_rank']

def test_load_sv_variants(real_populated_database, case_obj, sv_clinical_file):
    """Test to load a variant into a mongo database"""
    adapter = real_populated_database
    # GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0

    # WHEN loading a variant into the database
    rank_threshold = 0
    adapter.load_variants(
            case_obj=case_obj,
            variant_type='clinical',
            category='sv',
            rank_threshold=rank_threshold,
    )
    # THEN assert the variant is loaded

    assert adapter.variant_collection.find().count() > 0

    for variant in adapter.variant_collection.find():
        assert variant['rank_score'] >= rank_threshold
        assert variant['category'] == 'sv'
        assert variant['variant_rank']

def test_load_region(real_populated_database, case_obj, variant_clinical_file):
    """Test to load variants from a region into a mongo database"""
    adapter = real_populated_database
    # GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0

    # WHEN loading a variant into the database
    chrom = '1'
    start = 7847367
    end = 156126553
    adapter.load_variants(
            case_obj=case_obj,
            variant_type='clinical',
            category='snv',
            chrom=chrom,
            start=start,
            end=end,
    )
    # THEN assert all variants loaded are in the given region

    assert adapter.variant_collection.find().count() > 0

    for variant in adapter.variant_collection.find():
        assert variant['chromosome'] == chrom
        assert variant['position'] <= end
        assert variant['end'] >= start

def test_load_mitochondrie(real_populated_database, case_obj, variant_clinical_file):
    """Test that all variants from mt are loaded"""
    adapter = real_populated_database
    rank_threshold = 3
    
    # Check how many MT variants there are in file
    vcf_obj = VCF(variant_clinical_file)
    mt_variants = 0
    for variant in vcf_obj:
        if variant.CHROM == 'MT':
            mt_variants += 1
    
    # Make sure there are some MT variants
    assert mt_variants
    
    # GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0

    # WHEN loading a variant into the database

    adapter.load_variants(
            case_obj=case_obj,
            variant_type='clinical',
            category='snv',
            rank_threshold = rank_threshold,
    )
    # THEN assert all MT variants is loaded

    mt_variants_found = 0
    for variant in adapter.variant_collection.find():
        if variant['chromosome'] == 'MT':
            mt_variants_found += 1
        else:
            assert variant['rank_score'] >= rank_threshold

    assert mt_variants == mt_variants_found

def test_compounds_region(real_populated_database, case_obj):
    """When loading the variants not all variants will be loaded, only the ones that
       have a rank score above a treshold.
       This implies that some compounds will have the status 'not_loaded'=True.
       When loading all variants for a region then all variants should 
       have status 'not_loaded'=False.
    """
    adapter = real_populated_database
    variant_type = 'clinical'
    category = 'snv'
    ## GIVEN a database without any variants
    assert adapter.variant_collection.find().count() == 0
    
    ## WHEN loading a variant into the database
    adapter.load_variants(
            case_obj=case_obj,
            variant_type=variant_type,
            category=category,
            rank_threshold=-10
    )
    
    adapter.load_indexes()
    institute_obj = adapter.institute_collection.find_one()
    case_obj = adapter.case_collection.find_one()
    
    chrom = '1'
    start = 156112157
    end = 156152543
    
    query = adapter.build_query(
                case_id=case_obj['_id'],
                query={
                    'variant_type': variant_type,
                    'chrom': chrom,
                    'start': start,
                    'end': end,
                },
                category=category
            )
    ## THEN assert that there are variants with compounds without information
    variants_query = adapter.variant_collection.find(query)
    nr_comps = 0
    nr_variants = 0
    nr_not_loaded = 0
    genomic_regions = adapter.get_coding_intervals()
    for nr_variants,variant in enumerate(variants_query):
        for comp in variant.get('compounds',[]):
            nr_comps += 1
            if comp['not_loaded']:
                nr_not_loaded += 1
    
    assert nr_not_loaded > 0
    assert nr_variants > 0
    assert nr_comps > 0

    ## THEN when loading all variants in the region, assert that ALL the compounds are updated
    print('')
    adapter.load_variants(
            case_obj=case_obj,
            variant_type=variant_type,
            category=category,
            chrom=chrom,
            start=start,
            end=end
            )

    variants_query = adapter.variant_collection.find(query)
    nr_comps = 0
    nr_variants = 0
    for nr_variants,variant in enumerate(variants_query):
        for comp in variant.get('compounds',[]):
            nr_comps += 1
            # We know that all ar updated and loaded if this flag is set
            assert comp['not_loaded'] == False
    
    assert nr_variants > 0
    assert nr_comps > 0

def test_updated_panel(real_variant_database, case_obj):
    """Test if the annotated panels are correct on variant level when a gene is removed 
    from the panel. Ref #754
    
    In this test we need to update a gene panel by removing some genes and check that when loading
      variants they should not be annotated with the panel.
    
    """
    adapter = real_variant_database
    
    ## GIVEN an adapter with variants case and everything
    
    # Collect the hgnc_ids for all genes in the panel
    panel_hgnc_ids = set()
    # Get the panel object
    panel_name = case_obj['panels'][0]['panel_name']
    panel_obj = adapter.gene_panel(panel_name)
    for gene_obj in panel_obj['genes']:
        # Add the existing hgnc ids to the panel
        panel_hgnc_ids.add(gene_obj['hgnc_id'])
    
    # Loop over the variants and check that there are variants in
    # the genes from the panel
    variants = adapter.variants(case_id=case_obj['_id'])
    
    # Collect all present hgnc ids from the variants
    variant_hgnc_ids = set() 
    for variant in variants:
        # print(variant['hgnc_ids'])
        if variant.get('hgnc_ids'):
            for hgnc_id in variant['hgnc_ids']:
                if hgnc_id in panel_hgnc_ids:
                    # assert that the panel is annotated on the variant
                    assert panel_name in variant['panels']
                variant_hgnc_ids.add(hgnc_id)
    
    # Assert that there are data
    assert panel_hgnc_ids
    assert variant_hgnc_ids
    
    intersecting_ids = panel_hgnc_ids.intersection(variant_hgnc_ids)
    assert intersecting_ids
    
    # Create a new case and a new gene panel
    
    # Create and insert a new gene panel without the intersecting genes
    old_panel_genes = panel_obj['genes']
    panel_obj['genes'] = []
    for gene in old_panel_genes:
        if gene['hgnc_id'] not in intersecting_ids:
            panel_obj['genes'].append(gene)
    
    new_version = panel_obj['version'] + 1
    panel_obj['version'] = new_version
    panel_obj.pop('_id')
    
    adapter.panel_collection.insert_one(panel_obj)
    
    new_panel = adapter.panel_collection.find_one({'panel_name': panel_obj['panel_name'], 'version': new_version})
    new_panel_ids = set()
    
    for gene in new_panel['genes']:
        hgnc_id = gene['hgnc_id']
        new_panel_ids.add(hgnc_id)
        assert hgnc_id not in intersecting_ids
    
    # Create a new case with the new panel
    case_obj['_id'] = 'second_case'
    case_obj['panels'][0]['version'] = new_panel['version']
    case_obj['panels'][0]['panel_id'] =  new_panel['_id']

    # Insert the new case and the variants
    adapter._add_case(case_obj)
    
    new_caseobj = adapter.case_collection.find_one({'_id': 'second_case'})
    
    adapter.load_variants(
        new_caseobj, 
        variant_type='clinical', 
        category='snv',
        rank_threshold=-10,
        build='37'
    )
    
    # These are the new variants
    new_variants = adapter.variant_collection.find({'case_id': case_obj['_id']})
    
    nr_variants = 0
    for variant in new_variants:
        if variant.get('hgnc_ids'):
            for hgnc_id in variant['hgnc_ids']:
                if hgnc_id in intersecting_ids:
                    # assert that the panel is NOT annotated on the variant
                    # We removed the gener from the new panel
                    assert panel_name not in variant.get('panels',[])
                    nr_variants += 1
    assert nr_variants > 0



import logging
import click

LOG = logging.getLogger(__name__)


@click.command('diseases', short_help='Display all diseases')
@click.pass_context
def diseases(context):
    """Show all diseases in the database"""
    LOG.info("Running scout view diseases")
    adapter = context.obj['adapter']

    disease_objs = adapter.disease_terms()

    nr_diseases = disease_objs.count()
    if nr_diseases == 0:
        click.echo("No diseases found")
    else:
        click.echo("Disease")
        for disease_obj in adapter.disease_terms():
            click.echo("{0}".format(disease_obj['_id']))
        LOG.info("{0} diseases found".format(nr_diseases))



CLINSIG_MAP = {
    0: 'Uncertain significance',
    1: 'not provided',
    2: 'Benign',
    3: 'Likely benign',
    4: 'Likely pathogenic',
    5: 'Pathogenic',
    6: 'drug response',
    7: 'histocompatibility',
    255: 'other'
}
REV_CLINSIG_MAP = {
    'uncertain_significance': 0,
    'not_provided': 1,
    'benign': 2,
    'likely_benign': 3,
    'likely_pathogenic': 4,
    'pathogenic': 5,
    'drug_response': 6,
    'histocompatibility': 7,
    'other': 255
}



import logging
from datetime import datetime

from bson import ObjectId
import pymongo

from scout.constants import REV_ACMG_MAP

SANGER_OPTIONS = ['True positive', 'False positive', 'Not validated']

LOG = logging.getLogger(__name__)

class VariantEventHandler(object):
    """Class to handle variant events for the mongo adapter"""

    def pin_variant(self, institute, case, user, link, variant):
        """Create an event for pinning a variant.

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object

        Returns:
            updated_case
        """
        LOG.info("Creating event for pinning variant {0}".format(
            variant['display_name']))

        # add variant to list of pinned references in the case model
        updated_case = self.case_collection.find_one_and_update(
            {'_id': case['_id']},
            {'$push': {'suspects': variant['_id']}},
            return_document=pymongo.ReturnDocument.AFTER
        )

        kwargs = dict(
            institute=institute,
            case=case,
            user=user,
            link=link,
            verb='pin',
            variant=variant,
            subject=variant['display_name'],
        )
        self.create_event(category='variant', **kwargs)
        self.create_event(category='case', **kwargs)

        return updated_case

    def unpin_variant(self, institute, case, user, link, variant):
        """Create an event for unpinning a variant.

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object

        Returns:
            updated_case(dict)
        """
        LOG.info("Creating event for unpinning variant {0}".format(
            variant['display_name']))

        LOG.info("Remove variant from list of references in the case" \
                    " model")

        updated_case = self.case_collection.find_one_and_update(
            {'_id': case['_id']},
            {'$pull': {'suspects': variant['_id']}},
            return_document=pymongo.ReturnDocument.AFTER
        )

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='unpin',
            variant=variant,
            subject=variant['display_name'],
        )

        return updated_case

    def order_verification(self, institute, case, user, link, variant):
        """Create an event for a variant verification for a variant
        and an event for a variant verification for a case

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object

        Returns:
            updated_variant(dict)
        """
        LOG.info("Creating event for ordering validation for variant" \
                    " {0}".format(variant['display_name']))

        updated_variant = self.variant_collection.find_one_and_update(
            {'_id': variant['_id']},
            {'$set': {'sanger_ordered': True}},
            return_document=pymongo.ReturnDocument.AFTER
        )

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='sanger',
            variant=variant,
            subject=variant['display_name'],
        )

        LOG.info("Creating event for ordering sanger for case" \
                    " {0}".format(case['display_name']))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='case',
            verb='sanger',
            variant=variant,
            subject=variant['display_name'],
        )
        return updated_variant

    def cancel_verification(self, institute, case, user, link, variant):
        """Create an event for cancellation of a verification of a variant

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object

        Returns:
            updated_variant(dict)
        """
        LOG.info("Creating event for cancellation of ordering sanger for variant" \
                    " {0}".format(variant['display_name']))

        updated_variant = self.variant_collection.find_one_and_update(
            {'_id': variant['_id']},
            {'$set': {'sanger_ordered': False}},
            return_document=pymongo.ReturnDocument.AFTER
        )

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='cancel_sanger',
            variant=variant,
            subject=variant['display_name'],
        )

        LOG.info("Creating event for cancellation of ordering verification for case" \
                    " {0}".format(case['display_name']))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='case',
            verb='cancel_sanger',
            variant=variant,
            subject=variant['display_name'],
        )
        return updated_variant

    def sanger_ordered(self, institute_id=None, user_id=None):
        """Get all variants with validations ever ordered.

        Args:
            institute_id(str) : The id of an institute
            user_id(str) : The id of an user

        Returns:
            sanger_ordered(list) : a list of dictionaries, each with "case_id" as keys and list of variant ids as values
        """
        query = {'$match': {
                '$and': [
                    {'verb': 'sanger'},
                ],
            }}

        if institute_id:
            query['$match']['$and'].append({'institute': institute_id})
        if user_id:
            query['$match']['$and'].append({'user_id': user_id})

        # Get all sanger ordered variants grouped by case_id
        results = self.event_collection.aggregate([
            query,
            {'$group': {
                '_id': "$case",
                'vars': {'$addToSet' : '$variant_id'}
            }}
        ])

        sanger_ordered =  [item for item in results]
        return sanger_ordered

    def validate(self, institute, case, user, link, variant, validate_type):
        """Mark validation status for a variant.

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object
            validate_type(str): The outcome of validation.
                                choices=('True positive', 'False positive')

        Returns:
            updated_variant(dict)
        """
        if not validate_type in SANGER_OPTIONS:
            LOG.warning("Invalid validation string: %s", validate_type)
            LOG.info("Validation options: %s", ', '.join(SANGER_OPTIONS))
            return

        updated_variant = self.variant_collection.find_one_and_update(
            {'_id': variant['_id']},
            {'$set': {'validation': validate_type}},
            return_document=pymongo.ReturnDocument.AFTER
        )

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='validate',
            variant=variant,
            subject=variant['display_name'],
        )
        return updated_variant

    def mark_causative(self, institute, case, user, link, variant):
        """Create an event for marking a variant causative.

        Arguments:
          institute (dict): A Institute object
          case (dict): Case object
          user (dict): A User object
          link (str): The url to be used in the event
          variant (variant): A variant object

        Returns:
            updated_case(dict)
        """
        display_name = variant['display_name']
        LOG.info("Mark variant {0} as causative in the case {1}".format(
            display_name, case['display_name']))

        LOG.info("Adding variant to causatives in case {0}".format(
            case['display_name']))

        LOG.info("Marking case {0} as solved".format(
            case['display_name']))

        updated_case = self.case_collection.find_one_and_update(
            {'_id': case['_id']},
            {
                '$push': {'causatives': variant['_id']},
                '$set': {'status': 'solved'}
            },
            return_document=pymongo.ReturnDocument.AFTER
        )

        LOG.info("Creating case event for marking {0}" \
                    " causative".format(variant['display_name']))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='case',
            verb='mark_causative',
            variant=variant,
            subject=variant['display_name'],
        )

        LOG.info("Creating variant event for marking {0}" \
                    " causative".format(case['display_name']))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='mark_causative',
            variant=variant,
            subject=variant['display_name'],
        )
        return updated_case

    def unmark_causative(self, institute, case, user, link, variant):
        """Create an event for unmarking a variant causative

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object

        Returns:
            updated_case(dict)

        """
        display_name = variant['display_name']
        LOG.info("Remove variant {0} as causative in case {1}".format(
            display_name, case['display_name']))

        updated_case = self.case_collection.find_one_and_update(
            {'_id': case['_id']},
            {
                '$pull': {'causatives': variant['_id']},
            },
            return_document=pymongo.ReturnDocument.AFTER
        )

        # mark the case as active again
        if len(updated_case.get('causatives', [])) == 0:
            LOG.info("Marking case as 'active'")
            updated_case = self.case_collection.find_one_and_update(
                {'_id': case['_id']},
                {
                    '$set': {'status': 'active'}
                },
                return_document=pymongo.ReturnDocument.AFTER
            )

        LOG.info("Creating events for unmarking variant {0} " \
                    "causative".format(display_name))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='case',
            verb='unmark_causative',
            variant=variant,
            subject=variant['display_name'],
        )

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='unmark_causative',
            variant=variant,
            subject=variant['display_name'],
        )

        return updated_case

    def update_manual_rank(self, institute, case, user, link, variant,
                           manual_rank):
        """Create an event for updating the manual rank of a variant

          This function will create a event and update the manual rank
          of the variant.

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object
            manual_rank (int): The new manual rank

        Return:
            updated_variant

        """
        LOG.info("Creating event for updating manual rank for "
                    "variant {0}".format(variant['display_name']))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='manual_rank',
            variant=variant,
            subject=variant['display_name'],
        )

        if manual_rank:
            LOG.info("Setting manual rank to {0} for variant {1}"
                        .format(manual_rank, variant['display_name']))
            action = '$set'
        else:
            LOG.info("Reset manual rank from {0} for variant {1}"
                        .format(variant.get('manual_rank', 'NA'), variant['display_name']))
            action = '$unset'

        updated_variant = self.variant_collection.find_one_and_update(
            {'_id': variant['_id']},
            {action: {'manual_rank': manual_rank}},
            return_document=pymongo.ReturnDocument.AFTER
        )
        LOG.debug("Variant updated")
        return updated_variant

    def update_dismiss_variant(self, institute, case, user, link, variant,
                               dismiss_variant):
        """Create an event for updating the manual dismiss variant entry

          This function will create a event and update the dismiss variant
          field of the variant.

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object
            dismiss_variant (list): The new dismiss variant list

        Return:
            updated_variant

        """
        LOG.info("Creating event for updating dismiss variant for "
                    "variant {0}".format(variant['display_name']))

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='dismiss_variant',
            variant=variant,
            subject=variant['display_name'],
        )

        if dismiss_variant:
            LOG.info("Setting dismiss variant to {0} for variant {1}"
                        .format(dismiss_variant, variant['display_name']))
            action = '$set'
        else:
            LOG.info("Reset dismiss variant from {0} for variant {1}"
                        .format(variant['dismiss_variant'], variant['display_name']))
            action = '$unset'

        updated_variant = self.variant_collection.find_one_and_update(
            {'_id': variant['_id']},
            {action: {'dismiss_variant': dismiss_variant}},
            return_document=pymongo.ReturnDocument.AFTER
        )
        LOG.debug("Variant updated")
        return updated_variant

    def update_mosaic_tags(self, institute, case, user, link, variant,
                               mosaic_tags):
        """Create an event for updating the mosaicism tags for a variant

          This function will create a event and update the mosaicism tags of the variant.

        Arguments:
            institute (dict): A Institute object
            case (dict): Case object
            user (dict): A User object
            link (str): The url to be used in the event
            variant (dict): A variant object
            mosaic_tags (list): The new mosaic tags list

        Return:
            updated_variant

        """
        LOG.info("Creating event for updating mosaic tags for variant %s", variant['display_name'])

        self.create_event(
            institute=institute,
            case=case,
            user=user,
            link=link,
            category='variant',
            verb='mosaic_tags',
            variant=variant,
            subject=variant['display_name'],
        )

        if mosaic_tags:
            LOG.info("Setting mosaic tags to {0} for variant {1}"
                        .format(mosaic_tags, variant['display_name']))
            action = '$set'
        else:
            LOG.info("Reset dismiss variant from {0} for variant {1}"
                        .format(variant['mosaic_tags'], variant['display_name']))
            action = '$unset'

        updated_variant = self.variant_collection.find_one_and_update(
            {'_id': variant['_id']},
            {action: {'mosaic_tags': mosaic_tags}},
            return_document=pymongo.ReturnDocument.AFTER
        )
        LOG.debug("Variant updated")
        return updated_variant

    def update_acmg(self, institute_obj, case_obj, user_obj, link, variant_obj, acmg_str):
        """Create an event for updating the ACMG classification of a variant.

        Arguments:
            institute_obj (dict): A Institute object
            case_obj (dict): Case object
            user_obj (dict): A User object
            link (str): The url to be used in the event
            variant_obj (dict): A variant object
            acmg_str (str): The new ACMG classification string

        Returns:
            updated_variant
        """
        self.create_event(
            institute=institute_obj,
            case=case_obj,
            user=user_obj,
            link=link,
            category='variant',
            verb='acmg',
            variant=variant_obj,
            subject=variant_obj['display_name'],
        )
        LOG.info("Setting ACMG to {} for: {}".format(acmg_str, variant_obj['display_name']))

        if acmg_str is None:
            updated_variant = self.variant_collection.find_one_and_update(
                {'_id': variant_obj['_id']},
                {'$unset': {'acmg_classification': 1}},
                return_document=pymongo.ReturnDocument.AFTER
            )
        else:
            updated_variant = self.variant_collection.find_one_and_update(
                {'_id': variant_obj['_id']},
                {'$set': {'acmg_classification': REV_ACMG_MAP[acmg_str]}},
                return_document=pymongo.ReturnDocument.AFTER
            )

        LOG.debug("Variant updated")
        return updated_variant



from pprint import pprint as pp
from scout.load.panel import load_panel

def test_load_panel(gene_database, panel_info):
    # GIVEN an database with genes but no panels
    adapter = gene_database
    assert adapter.gene_panels().count() == 0
    
    # WHEN loading a gene panel
    load_panel(
        panel_path=panel_info['file'], 
        adapter=adapter,
        institute=panel_info['institute'], 
        panel_id=panel_info['panel_name'], 
        panel_type=panel_info['type'], 
        version=panel_info['version'], 
        display_name=panel_info['full_name']
    )
    
    # THEN make sure that the panel is loaded
    
    assert adapter.gene_panel(panel_id=panel_info['panel_name'])


from pymongo import (IndexModel, ASCENDING, DESCENDING, TEXT)

INDEXES = {
    'hgnc_gene': [
        IndexModel([
            ('build', ASCENDING),
            ('chromosome', ASCENDING)],
            name="build_chromosome",
            background=True,
            ),
         IndexModel([
            ('build', ASCENDING),
            ('hgnc_id', ASCENDING)],
            name="build_hgncid",
            background=True,
            ),
        IndexModel([
            ('build', ASCENDING),
            ('aliases', ASCENDING)],
            name="build_aliases",
            background=True,
            ),
    ],
    'variant': [
        IndexModel([
            ('case_id', ASCENDING),
            ('category', ASCENDING),
            ('rank_score', DESCENDING)],
            name="caseid_rankscore",
            background=True,
            ),
        IndexModel([
            ('case_id', ASCENDING),
            ('category', ASCENDING),
            ('variant_rank', ASCENDING)],
            name="caseid_variantrank",
            background=True,
            ),
        IndexModel([
            ('case_id', ASCENDING),
            ('category', ASCENDING),
            ('variant_type', ASCENDING),
            ('rank_score', DESCENDING)],
            name="caseid_category_varianttype_rankscore",
            background=True,
            ),
        IndexModel([
            ('case_id', ASCENDING),
            ('category', ASCENDING),
            ('variant_id', ASCENDING)],
            name="caseid_variantid",
            background=True,
            ),
        IndexModel([
            ('case_id', ASCENDING),
            ('category', ASCENDING),
            ('variant_type', ASCENDING),
            ('chromosome', ASCENDING),
            ('start', ASCENDING),
            ('end', ASCENDING),],
            name="caseid_category_chromosome_start_end",
            background=True,
            ),
        IndexModel([
            ('sanger_ordered', ASCENDING),
            ],
            name="sanger",
            background=True,
            sparse=True,
            ),
        
    ],
    'hpo_term': [
        IndexModel([
            ('description', ASCENDING)],
            name="description"),
        IndexModel([
            ('description', TEXT)],
            default_language='english',
            name="description_text"),
        IndexModel([
            ('hpo_number', ASCENDING)],
            name="number"),
    ],
    'event': [
        IndexModel([
            ('category', ASCENDING),
            ('verb', ASCENDING),
            ],
            name="category_verb",
            ),
        IndexModel([
            ('variant_id', ASCENDING),
            ],
            name="variant_id",
            ),
    ],
    'transcript': [
        IndexModel([
            ('build', ASCENDING),
            ('hgnc_id', ASCENDING),
            ('length', DESCENDING)],
            name="hgncid_length",
            background=True,
            ),
    ],
    'exon': [
        IndexModel([
            ('build', ASCENDING),
            ('hgnc_id', ASCENDING)],
            name="build_hgncid",
            background=True,
            ),
    ],
    'hpo_term': [
        IndexModel([
            ('hpo_number', ASCENDING)],
            name="number",
            background=True,
            ),
    ],
    'case': [
        IndexModel([
            ('synopsis', TEXT)],
            default_language='english',
            name="synopsis_text"),
    ],
}



# -*- coding: utf-8 -*-
import logging

from pprint import pprint as pp
from datetime import datetime

from scout.utils.date import get_date
from scout.utils.handle import get_file_handle
from scout.utils.link import get_correct_ids

from .omim import get_mim_genes

LOG = logging.getLogger(__name__)

VALID_MODELS = ('AR','AD','MT','XD','XR','X','Y')

MODELS_MAP = {
    'monoallelic_not_imprinted': ['AD'],
    'monoallelic_maternally_imprinted': ['AD'],
    'monoallelic_paternally_imprinted': ['AD'],
    'monoallelic': ['AD'],
    'biallelic': ['AR'],
    'monoallelic_and_biallelic': ['AD','AR'] ,
    'monoallelic_and_more_severe_biallelic': ['AD','AR'],
    'xlinked_biallelic': ['XR'],
    'xlinked_monoallelic': ['XD'],
    'mitochondrial': ['MT'],
    'unknown': [],
}

INCOMPLETE_PENETRANCE_MAP = {
    'unknown': None,
    'Complete': None,
    'Incomplete': True
}


def get_panel_info(panel_lines=None, panel_id=None, institute=None, version=None, date=None,
                   display_name=None):
    """Parse metadata for a gene panel

    For historical reasons it is possible to include all information about a gene panel in the
    header of a panel file. This function parses the header.

    Args:
        panel_lines(iterable(str))

    Returns:
        panel_info(dict): Dictionary with panel information
    """
    panel_info = {
        'panel_id': panel_id,
        'institute': institute,
        'version': version,
        'date': date,
        'display_name': display_name,
    }

    if panel_lines:
        for line in panel_lines:
            line = line.rstrip()
            if not line.startswith('##'):
                break
        
            info = line[2:].split('=')
            field = info[0]
            value = info[1]
        
        
            if not panel_info.get(field):
                panel_info[field] = value

    panel_info['date'] = get_date(panel_info['date'])

    return panel_info


def parse_gene(gene_info):
    """Parse a gene line with information from a panel file

        Args:
            gene_info(dict): dictionary with gene info

        Returns:
            gene(dict): A dictionary with the gene information
                {
                'hgnc_id': int,
                'hgnc_symbol': str,
                'disease_associated_transcripts': list(str),
                'inheritance_models': list(str),
                'mosaicism': bool,
                'reduced_penetrance': bool,
                'database_entry_version': str,
                }

    """
    gene = {}
    # This is either hgnc id or hgnc symbol
    identifier = None

    hgnc_id = None
    try:
        if 'hgnc_id' in gene_info:
            hgnc_id = int(gene_info['hgnc_id'])
        elif 'hgnc_idnumber' in gene_info:
            hgnc_id = int(gene_info['hgnc_idnumber'])
        elif 'hgncid' in gene_info:
            hgnc_id = int(gene_info['hgncid'])
    except ValueError as e:
        raise SyntaxError("Invalid hgnc id: {0}".format(hgnc_id))

    gene['hgnc_id'] = hgnc_id
    identifier = hgnc_id

    hgnc_symbol = None
    if 'hgnc_symbol' in gene_info:
        hgnc_symbol = gene_info['hgnc_symbol']
    elif 'hgncsymbol' in gene_info:
        hgnc_symbol = gene_info['hgncsymbol']
    elif 'symbol' in gene_info:
        hgnc_symbol = gene_info['symbol']

    gene['hgnc_symbol'] = hgnc_symbol

    if not identifier:
        if hgnc_symbol:
            identifier = hgnc_symbol
        else:
            raise SyntaxError("No gene identifier could be found")
    gene['identifier'] = identifier
    # Disease associated transcripts is a ','-separated list of
    # manually curated transcripts
    transcripts = ""
    if 'disease_associated_transcripts' in gene_info:
        transcripts = gene_info['disease_associated_transcripts']
    elif 'disease_associated_transcript' in gene_info:
        transcripts = gene_info['disease_associated_transcript']
    elif 'transcripts' in gene_info:
        transcripts = gene_info['transcripts']

    gene['transcripts'] = [
            transcript.strip() for transcript in
            transcripts.split(',') if transcript
        ]

    # Genetic disease models is a ','-separated list of manually curated
    # inheritance patterns that are followed for a gene
    models = ""
    if 'genetic_disease_models' in gene_info:
        models = gene_info['genetic_disease_models']
    elif 'genetic_disease_model' in gene_info:
        models = gene_info['genetic_disease_model']
    elif 'inheritance_models' in gene_info:
        models = gene_info['inheritance_models']
    elif 'genetic_inheritance_models' in gene_info:
        models = gene_info['genetic_inheritance_models']

    gene['inheritance_models'] = [
        model.strip() for model in models.split(',')
        if model.strip() in VALID_MODELS
    ]

    # If a gene is known to be associated with mosaicism this is annotated
    gene['mosaicism'] = True if gene_info.get('mosaicism') else False

    # If a gene is known to have reduced penetrance this is annotated
    gene['reduced_penetrance'] = True if gene_info.get('reduced_penetrance') else False

    # The database entry version is a way to track when a a gene was added or
    # modified, optional
    gene['database_entry_version'] = gene_info.get('database_entry_version')

    return gene

def parse_genes(gene_lines):
    """Parse a file with genes and return the hgnc ids

    Args:
        gene_lines(iterable(str)): Stream with genes

    Returns:
        genes(list(dict)): Dictionaries with relevant gene info
    """
    genes = []
    header = []
    hgnc_identifiers = set()
    delimiter = '\t'
    # This can be '\t' or ';'
    delimiters = ['\t', ' ', ';']

    # There are files that have '#' to indicate headers
    # There are some files that start with a header line without
    # any special symbol
    for i,line in enumerate(gene_lines):
        line = line.rstrip()
        if not len(line) > 0:
            continue
        if line.startswith('#'):
            if not line.startswith('##'):
                # We need to try delimiters
                # We prefer ';' or '\t' byt should accept ' '
                line_length = 0
                delimiter = None
                for alt in delimiters:
                    head_line = line.split(alt)
                    if len(head_line) > line_length:
                        line_length = len(head_line)
                        delimiter = alt

                header = [word.lower() for word in line[1:].split(delimiter)]
        else:
            # If no header symbol(#) assume first line is header
            if i == 0:
                line_length = 0
                for alt in delimiters:
                    head_line = line.split(alt)
                    if len(head_line) > line_length:
                        line_length = len(head_line)
                        delimiter = alt
                
                if ('hgnc' in line or 'HGNC' in line):
                    header = [word.lower() for word in line.split(delimiter)]
                    continue
                # If first line is not a header try to sniff what the first
                # columns holds
                if line.split(delimiter)[0].isdigit():
                    header = ['hgnc_id']
                else:
                    header = ['hgnc_symbol']

            splitted_line = line.split(delimiter)
            gene_info = dict(zip(header, splitted_line))

            # There are cases when excel exports empty lines that looks like
            # ;;;;;;;. This is a exception to handle these
            info_found = False
            for key in gene_info:
                if gene_info[key]:
                    info_found = True
                    break
            # If no info was found we skip that line
            if not info_found:
                continue

            try:
                gene = parse_gene(gene_info)
            except Exception as e:
                LOG.warning(e)
                raise SyntaxError("Line {0} is malformed".format(i + 1))

            identifier = gene.pop('identifier')

            if not identifier in hgnc_identifiers:
                hgnc_identifiers.add(identifier)
                genes.append(gene)

    return genes


def parse_gene_panel(path, institute='cust000', panel_id='test', panel_type='clinical', date=datetime.now(), 
                     version=1.0, display_name=None, genes = None):
    """Parse the panel info and return a gene panel

        Args:
            path(str): Path to panel file
            institute(str): Name of institute that owns the panel
            panel_id(str): Panel id
            date(datetime.datetime): Date of creation
            version(float)
            full_name(str): Option to have a long name

        Returns:
            gene_panel(dict)
    """
    LOG.info("Parsing gene panel %s", panel_id)
    gene_panel = {}

    gene_panel['path'] = path
    gene_panel['type'] = panel_type
    gene_panel['date'] = date
    gene_panel['panel_id'] = panel_id
    gene_panel['institute'] = institute
    version = version or 1.0
    gene_panel['version'] = float(version)
    gene_panel['display_name'] = display_name or panel_id

    if not path:
        panel_handle = genes
    else:
        panel_handle = get_file_handle(gene_panel['path'])
    gene_panel['genes'] = parse_genes(gene_lines=panel_handle)

    return gene_panel

def parse_panel_app_gene(app_gene, hgnc_map):
    """Parse a panel app formated gene
    
    Args:
        app_gene(dict): Dict with panel app info
        hgnc_map(dict): Map from hgnc_symbol to hgnc_id
    
    Returns:
        gene_info(dict): Scout infromation
    """
    gene_info = {}
    confidence_level = app_gene['LevelOfConfidence']
    # Return empty gene if not confident gene
    if not confidence_level == 'HighEvidence':
        return gene_info
    
    hgnc_symbol = app_gene['GeneSymbol']
    # Returns a set of hgnc ids
    hgnc_ids = get_correct_ids(hgnc_symbol, hgnc_map)
    if not hgnc_ids:
        LOG.warning("Gene %s does not exist in database. Skipping gene...", hgnc_symbol)
        return gene_info
    
    if len(hgnc_ids) > 1:
        LOG.warning("Gene %s has unclear identifier. Choose random id", hgnc_symbol)

    gene_info['hgnc_symbol'] = hgnc_symbol
    for hgnc_id in hgnc_ids:
        gene_info['hgnc_id'] = hgnc_id

    gene_info['reduced_penetrance'] = INCOMPLETE_PENETRANCE_MAP.get(app_gene['Penetrance'])

    inheritance_models = []
    for model in MODELS_MAP.get(app_gene['ModeOfInheritance'],[]):
        inheritance_models.append(model)
    
    gene_info['inheritance_models'] = inheritance_models
    
    return gene_info

def parse_panel_app_panel(panel_info, hgnc_map, institute='cust000', panel_type='clinical'):
    """Parse a PanelApp panel
    
    Args:
        panel_info(dict)
        hgnc_map(dict): Map from symbol to hgnc ids
        institute(str)
        panel_type(str)
    
    Returns:
        gene_panel(dict)
    """
    date_format = "%Y-%m-%dT%H:%M:%S.%f"
    
    gene_panel = {}
    gene_panel['version'] = float(panel_info['version'])
    gene_panel['date'] = get_date(panel_info['Created'][:-1], date_format=date_format)
    gene_panel['display_name'] = panel_info['SpecificDiseaseName']
    gene_panel['institute'] = institute
    gene_panel['panel_type'] = panel_type
    
    LOG.info("Parsing panel %s", gene_panel['display_name'])
    
    gene_panel['genes'] = []
    
    nr_low_confidence = 1
    nr_genes = 0
    for nr_genes, gene in enumerate(panel_info['Genes'],1):
        gene_info = parse_panel_app_gene(gene, hgnc_map)
        if not gene_info:
            nr_low_confidence += 1
            continue
        gene_panel['genes'].append(gene_info)
    
    LOG.info("Number of genes in panel %s", nr_genes)
    LOG.info("Number of low confidence genes in panel %s", nr_low_confidence)
    
    return gene_panel

def get_omim_panel_genes(genemap2_lines, mim2gene_lines, alias_genes):
    """Return all genes that should be included in the OMIM-AUTO panel
    Return the hgnc symbols
    
    Genes that have at least one 'established' or 'provisional' phenotype connection
    are included in the gene panel
    
    Args:
        genemap2_lines(iterable)
        mim2gene_lines(iterable)
        alias_genes(dict): A dictionary that maps hgnc_symbol to hgnc_id
    
    Yields:
        hgnc_symbol(str)
    """
    parsed_genes = get_mim_genes(genemap2_lines, mim2gene_lines)
    
    STATUS_TO_ADD = set(['established', 'provisional'])
    
    for hgnc_symbol in parsed_genes:
        try:
            gene = parsed_genes[hgnc_symbol]
            keep = False
            for phenotype_info in gene.get('phenotypes',[]):
                if phenotype_info['status'] in STATUS_TO_ADD:
                    keep = True
                    break
            if keep:
                hgnc_id_info = alias_genes.get(hgnc_symbol)
                if not hgnc_id_info:
                    for symbol in gene.get('hgnc_symbols', []):
                        if symbol in alias_genes:
                            hgnc_id_info = alias_genes[symbol]
                            break

                if hgnc_id_info:
                    yield {
                    'hgnc_id': hgnc_id_info['true'],
                    'hgnc_symbol': hgnc_symbol,
                    }
                else:
                    LOG.warning("Gene symbol %s does not exist", hgnc_symbol)
                    
        except KeyError:
            pass
    




from scout.parse.variant.transcript import parse_transcripts

def test_parse_transcripts(cyvcf2_variant):
    ## GIVEN some transcript information and a vep header
    csq_entry = "C|missense_variant|MODERATE|POC1A|ENSG00000164087|"\
    "Transcript|ENST00000296484|protein_coding|4/11||ENST00000296484.2"\
    ":c.322A>G|ENSP00000296484.2:p.Ser108Gly|362|322|108|S/G|Agt/Ggt"\
    "|||-1|HGNC|24488||CCDS2846.1|ENSP00000296484|POC1A_HUMAN|"\
    "B2RDV4_HUMAN|UPI000045711C|deleterious|possibly_damaging|"\
    "PROSITE_profiles:PS50082&PROSITE_profiles:PS50294&hmmpanther"\
    ":PTHR22847:SF319&hmmpanther:PTHR22847&Gene3D:2.130.10.10&"\
    "Pfam_domain:PF00400&SMART_domains:SM00320&Superfamily_domains:"\
    "SSF50978|||||,C|missense_variant|MODERATE|POC1A|ENSG00000164087"\
    "|Transcript|ENST00000394970|protein_coding|4/10||"\
    "ENST00000394970.2:c.322A>G|ENSP00000378421.2:p.Ser108Gly|640"\
    "|322|108|S/G|Agt/Ggt|||-1|HGNC|24488||CCDS54592.1|ENSP00000378421"\
    "|POC1A_HUMAN||UPI00006633C6|deleterious|benign|"\
    "Superfamily_domains:SSF50978&SMART_domains:SM00320&Pfam_domain"\
    ":PF00400&Gene3D:2.130.10.10&hmmpanther:PTHR22847&hmmpanther:"\
    "PTHR22847:SF319&PROSITE_profiles:PS50082&PROSITE_profiles:PS50294"\
    "|||||,C|missense_variant|MODERATE|POC1A|ENSG00000164087|"\
    "Transcript|ENST00000474012|protein_coding|4/11||ENST00000474012"\
    ".1:c.208A>G|ENSP00000418968.1:p.Ser70Gly|425|208|70|S/G|Agt"\
    "/Ggt|||-1|HGNC|24488||CCDS54591.1|ENSP00000418968|POC1A_HUMAN|"\
    "B2RDV4_HUMAN|UPI0000E1FCF5|tolerated|possibly_damaging|Superfami"\
    "ly_domains:SSF50978&SMART_domains:SM00320&Gene3D:2.130.10.10&"\
    "Pfam_domain:PF00400&hmmpanther:PTHR22847:SF319&hmmpanther:"\
    "PTHR22847&PROSITE_profiles:PS50294&PROSITE_profiles:PS50082|||||"
    
    csq_header = "Allele|Consequence|IMPACT|SYMBOL|Gene|Feature_type|"\
    "Feature|BIOTYPE|EXON|INTRON|HGVSc|HGVSp|cDNA_position|CDS_position|"\
    "Protein_position|Amino_acids|Codons|Existing_variation|DISTANCE|"\
    "STRAND|SYMBOL_SOURCE|HGNC_ID|TSL|CCDS|ENSP|SWISSPROT|TREMBL|UNIPARC|"\
    "SIFT|PolyPhen|DOMAINS|HGVS_OFFSET|MOTIF_NAME|MOTIF_POS|HIGH_INF_POS|"\
    "MOTIF_SCORE_CHANGE"
    
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) for entry in csq_entry.split(',')]
    
    
    ## WHEN parsing the transcript
    transcripts = parse_transcripts(raw_transcripts)
    for transcript in transcripts:
    ## THEN assert that some information was correct
        if transcript['transcript_id'] == 'ENST00000296484':
            assert transcript['sift_prediction'] == 'deleterious'
            assert transcript['functional_annotations'] == ['missense_variant']

def test_parse_functional_annotation():
    csq_header = "Allele|Consequence"
    csq_entry = "C|missense_variant"
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) for entry in csq_entry.split(',')]

    transcripts = parse_transcripts(raw_transcripts)
    
    for transcript in transcripts:
        assert transcript['functional_annotations'] == ['missense_variant']

def test_parse_optional_hgnc_annotation():
    ## GIVEN a transcript with the optional hgnc annotation
    csq_header = "Allele|Consequence|HGNC_ID"
    csq_entry = "C|missense_variant|HGNC:10001"
    
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['functional_annotations'] == ['missense_variant']
        assert transcript['hgnc_id'] == 10001

def test_parse_regular_hgnc_annotation():
    ## GIVEN a transcript with the regular hgnc annotation
    csq_header = "Allele|Consequence|HGNC_ID"
    csq_entry = "C|missense_variant|10001"
    
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) 
                            for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['functional_annotations'] == ['missense_variant']
        assert transcript['hgnc_id'] == 10001

def test_parse_vep_freq_thousand_g():
    ## GIVEN a transcript with the 1000G frequency
    freq = 0.01
    csq_header = "Allele|Consequence|1000GAF"
    csq_entry = "C|missense_variant|{0}".format(freq)

    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) 
                            for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['thousand_g_maf'] == freq

def test_parse_vep_freq_thousand_g_alt():
    ## GIVEN a transcript with the 1000G frequency
    freq = 0.01
    csq_header = "Allele|Consequence|AF"
    csq_entry = "C|missense_variant|{0}".format(freq)
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) 
                            for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['thousand_g_maf'] == freq

def test_parse_vep_freq_gnomad():
    ## GIVEN a transcript with the 1000G frequency
    freq = 0.01
    csq_header = "Allele|Consequence|gnomAD_AF"
    csq_entry = "C|missense_variant|{0}".format(freq)
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) 
                            for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['gnomad_maf'] == freq

def test_parse_vep_freq_exac():
    ## GIVEN a transcript with the 1000G frequency
    freq = 0.01
    csq_header = "Allele|Consequence|EXAC_MAX_AF"
    csq_entry = "C|missense_variant|{0}".format(freq)
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) 
                            for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['exac_max'] == freq

def test_parse_vep_freq_thousand_g_max():
    ## GIVEN a transcript with the 1000G frequency
    freqs = [0.01, 0.001]
    csq_header = "Allele|Consequence|AFR_AF|AMR_AF"
    csq_entry = "C|missense_variant|{0}|{1}".format(freqs[0], freqs[1])
    header = [word.upper() for word in csq_header.split('|')]
    
    raw_transcripts = [dict(zip(header, entry.split('|'))) 
                            for entry in csq_entry.split(',')]

    ## WHEN parsing the transcripts
    transcripts = parse_transcripts(raw_transcripts)
    
    ## THEN assert that the hgnc annotation is parsed correct
    for transcript in transcripts:
        assert transcript['thousandg_max'] == max(freqs)




ANALYSIS_TYPES = ('wgs', 'wes', 'mixed', 'unknown')

SEX_MAP = {1: 'male', 2: 'female', 'other': 'unknown', 0: 'unknown', 
          '1': 'male', '2': 'female', '0': 'unknown'}
REV_SEX_MAP = {'male':'1', 'female': '2', 'unknown': '0'}

PHENOTYPE_MAP = {1: 'unaffected', 2: 'affected', 0: 'unknown', -9: 'unknown'}
REV_PHENOTYPE_MAP = {value: key for key, value in PHENOTYPE_MAP.items()}


CASE_STATUSES = ("prioritized", "inactive", "active", "solved", "archived")

VERBS_MAP = {
    "assign": "was assigned to",
    "unassign": "was unassigned from",
    "status": "updated the status for",
    "comment": "commented on",
    "synopsis": "updated synopsis for",
    "pin": "pinned variant",
    "unpin": "removed pinned variant",
    "sanger": "ordered sanger sequencing for",
    "cancel_sanger": "cancelled sanger order for",
    "archive": "archived",
    "open_research": "opened research mode for",
    "mark_causative": "marked causative for",
    "unmark_causative": "unmarked causative for",
    "manual_rank": "updated manual rank for",
    "add_phenotype": "added HPO term for",
    "remove_phenotype": "removed HPO term for",
    "add_case": "added case",
    "update_case": "updated case",
    "check_case": "marked case as",
    "share": "shared case with",
    "unshare": "revoked access for",
    "rerun": "requested rerun of",
    "validate": "marked validation status for",
    "update_diagnosis": "updated diagnosis for",
    "add_cohort": "updated cohort for",
    "remove_cohort": "removed cohort for",
    "acmg": "updated ACMG classification for",
    "dismiss_variant": "Dismissed variant for",
    "mosaic_tags": "Updated mosaic tags for",
    "update_default_panels": "updated default panels for",
}

VERBS = list(VERBS_MAP.keys())



PHENOTYPE_GROUPS = {
    'HP:0001298': {
        'name': 'Encephalopathy',
        'abbr': 'ENC'
    },
    'HP:0012759': {
        'name': 'Neurodevelopmental abnormality',
        'abbr': 'NDEV'
    },
    'HP:0001250': {
        'name': 'Seizures',
        'abbr': 'EP'
    },
    'HP:0100022': {
        'name': 'Abnormality of movement',
        'abbr': 'MOVE'
    },
    'HP:0000707': {
        'name': 'Neurology, other',
        'abbr': 'NEUR'
    },
    'HP:0003011': {
        'name': 'Abnormality of the musculature',
        'abbr': 'MUSC'
    },
    'HP:0001638': {
        'name': 'Cardiomyopathy',
        'abbr': 'CARD'
    },
    'HP:0001507': {
        'name': 'Growth abnormality',
        'abbr': 'GROW'
    },
    'HP:0001392': {
        'name': 'Abnormality of the liver',
        'abbr': 'LIV'
    },
    'HP:0011458': {
        'name': 'Abdominal symptom',
        'abbr': 'GI'
    },
    'HP:0012373': {
        'name': 'Abnormal eye physiology',
        'abbr': 'EYE'
    },
    'HP:0000077': {
        'name': 'Abnormality of the kidney',
        'abbr': 'KIDN'
    },
    'HP:0000951': {
        'name': 'Abnormality of the skin',
        'abbr': 'SKIN'
    },
    'HP:0001939': {
        'name': 'Abnormality of metabolism/homeostasis',
        'abbr': 'METAB'
    },
    'HP:0000118': {
        'name': 'As yet undefined/to be added',
        'abbr': 'UND'
    },
    'HP:0002011': {
        'name': 'Abnormal CNS morphology',
        'abbr': 'MRI'
    }
}

COHORT_TAGS = [
    'endo',
    'mito',
    'ketogenic diet',
    'pedhep',
    'other',
]


import logging

import click

LOG = logging.getLogger(__name__)

@click.command('panel', short_help='Delete a gene panel')
@click.option('--panel-id', 
    help="The panel identifier name",
    required=True
)
@click.option('-v', '--version',
    type=float,
)
@click.pass_context
def panel(context, panel_id, version):
    """Delete a version of a gene panel or all versions of a gene panel"""
    LOG.info("Running scout delete panel")
    adapter = context.obj['adapter']

    panel_objs = adapter.gene_panels(panel_id=panel_id, version=version)
    if panel_objs.count() == 0:
        LOG.info("No panels found")
        
    for panel_obj in panel_objs:
        adapter.delete_panel(panel_obj)

# @click.command('users', short_help='Display users')
# @click.pass_context
# def users(context):
#     """Show all users in the database"""
#     LOG.info("Running scout view users")
#     adapter = context.obj['adapter']
#
#     ## TODO add a User interface to the adapter
#     for user_obj in User.objects():
#         click.echo(user_obj['name'])
#
# @click.command('institutes', short_help='Display institutes')
# @click.pass_context
# def institutes(context):
#     """Show all institutes in the database"""
#     LOG.info("Running scout view institutes")
#     adapter = context.obj['adapter']
#
#     for institute_obj in adapter.institutes():
#         click.echo(institute_obj['internal_id'])


@click.command('index', short_help='Delete all indexes')
@click.pass_context
def index(context):
    """Delete all indexes in the database"""
    LOG.info("Running scout delete index")
    adapter = context.obj['adapter']
    
    for collection in adapter.db.collection_names():
        adapter.db[collection].drop_indexes()
    LOG.info("All indexes deleted")


@click.command('user', short_help='Delete a user')
@click.option('-m', '--mail', required=True)
@click.pass_context
def user(context, mail):
    """Delete a user from the database"""
    LOG.info("Running scout delete user")
    adapter = context.obj['adapter']
    user_obj = adapter.user(mail)
    if not user_obj:
        LOG.warning("User {0} could not be found in database".format(mail))
    else:
        adapter.delete_user(mail)


@click.command('genes', short_help='Delete genes')
@click.option('-b', 'build', type=click.Choice(['37', '38']))
@click.pass_context
def genes(context, build):
    """Delete all genes in the database"""
    LOG.info("Running scout delete genes")
    adapter = context.obj['adapter']

    if build:
        LOG.info("Dropping genes collection for build: %s", build)
    else:
        LOG.info("Dropping genes collection")
        adapter.drop_genes()

@click.command('exons', short_help='Delete exons')
@click.option('-b', 'build', type=click.Choice(['37', '38']))
@click.pass_context
def exons(context, build):
    """Delete all exons in the database"""
    LOG.info("Running scout delete exons")
    adapter = context.obj['adapter']

    adapter.drop_exons(build)


@click.command('case', short_help='Delete a case')
@click.option('-i', '--institute', help='institute id of related cases')
@click.option('-c', '--case-id')
@click.option('-d', '--display-name')
@click.pass_context
def case(context, institute, case_id, display_name):
    """Delete a case and it's variants from the database"""
    adapter = context.obj['adapter']
    if not (case_id or display_name):
        click.echo("Please specify what case to delete")
        context.abort()

    if display_name:
        if not institute:
            click.echo("Please specify the owner of the case that should be "
                       "deleted with flag '-i/--institute'.")
            context.abort()
        case_id = "{0}-{1}".format(institute, display_name)

    LOG.info("Running deleting case {0}".format(case_id))
    case = adapter.delete_case(
        case_id=case_id,
        institute_id=institute,
        display_name=display_name
    )

    if case.deleted_count == 1:
        adapter.delete_variants(case_id=case_id, variant_type='clinical')
        adapter.delete_variants(case_id=case_id, variant_type='research')
    else:
        LOG.warning("Case does not exist in database")
        context.abort()

# @click.command('diseases', short_help='Display all diseases')
# @click.pass_context
# def diseases(context):
#     """Show all diseases in the database"""
#     LOG.info("Running scout view diseases")
#     adapter = context.obj['adapter']
#
#     click.echo("Disease")
#     for disease_obj in adapter.disease_terms():
#         click.echo("{0}:{1}".format(
#             disease_obj['source'],
#             disease_obj['disease_id'],
#         ))
#
# @click.command('hpo', short_help='Display all hpo terms')
# @click.pass_context
# def hpo(context):
#     """Show all hpo terms in the database"""
#     LOG.info("Running scout view hpo")
#     adapter = context.obj['adapter']
#
#     click.echo("hpo_id\tdescription")
#     for hpo_obj in adapter.hpo_terms():
#         click.echo("{0}\t{1}".format(
#             hpo_obj.hpo_id,
#             hpo_obj.description,
#         ))


@click.group()
@click.pass_context
def delete(context):
    """
    Delete objects from the database.
    """
    pass


delete.add_command(genes)
delete.add_command(case)
delete.add_command(user)
delete.add_command(index)
delete.add_command(panel)
delete.add_command(exons)



import logging
import click
import urllib.request

from pprint import pprint as pp

from scout.parse.omim import (parse_genemap2, parse_mim2gene, parse_omim_morbid, parse_mim_titles,
                              get_mim_genes, get_mim_phenotypes)

LOG = logging.getLogger(__name__)

@click.command('omim', short_help='Update omim gene panel')
@click.option('--api-key', help='Specify the api key')
@click.option('--institute', 
    help='Specify the owner of the omim panel',
    default='cust002',
    show_default=True,
)
@click.pass_context
def omim(context, api_key, institute):
    """
    Update the automate generated omim gene panel in the database.
    """
    LOG.info("Running scout update omim")
    adapter = context.obj['adapter']
    
    api_key = api_key or context.obj.get('omim_api_key')
    if not api_key:
        LOG.warning("Please provide a omim api key to load the omim gene panel")
        context.abort()
    
    institute_obj = adapter.institute(institute)
    if not institute_obj:
        LOG.info("Institute %s could not be found in database", institute)
        LOG.warning("Please specify an existing institute")
        context.abort()

    try:
        adapter.load_omim_panel(api_key, institute=institute)
    except Exception as err:
        LOG.error(err)
        context.abort()
    
    


from cyvcf2 import VCF

from scout.parse.variant.genotype import (parse_genotypes, parse_genotype, GENOTYPE_MAP)

one_cnvnator = "tests/parse/vcfs/one_cnvnator.vcf"

def test_parse_genotype(variants):
    ## GIVEN a set of variants and information about the individuals
    ind_ids = variants.samples
    ind_positions = {}
    individuals = {}
    for i, ind in enumerate(ind_ids):
        ind_positions[ind] = i
        individuals[ind] = {
            'individual_id': ind,
            'display_name': ind
        }

    ## WHEN parsing the variants genotypes
    for variant in variants:
        for ind_id in ind_ids:
            pos = ind_positions[ind_id]
            genotype = parse_genotype(
                            variant=variant,
                            ind=individuals[ind_id],
                            pos=pos
                        )
    ## THEN assert genotypes are parsed correct
            vcf_genotype = variant.genotypes[pos]
            gt_call = "{0}/{1}".format(
                GENOTYPE_MAP[vcf_genotype[0]],
                GENOTYPE_MAP[vcf_genotype[1]]
                )

            vcf_read_depth = int(variant.gt_depths[pos])
            vcf_quality = float(variant.gt_quals[pos])

            if vcf_read_depth != -1:
                assert genotype['genotype_call'] == gt_call
                assert genotype['read_depth'] == vcf_read_depth
                assert genotype['genotype_quality'] == vcf_quality

def test_parse_genotypes(variants):
    ## GIVEN a set of variants and information about the individuals
    ind_ids = variants.samples
    ind_positions = {}
    individuals = {}
    case = {}
    for i, ind in enumerate(ind_ids):
        ind_positions[ind] = i
        individuals[ind] = {
            'individual_id': ind,
            'display_name': ind
        }
    case['individuals'] = [individuals[ind_id] for ind_id in individuals]
    ## WHEN parsing the variants genotypes
    for variant in variants:

    ## THEN assert genotypes are parsed correct

        genotypes = parse_genotypes(variant, case['individuals'], ind_positions)
        assert len(genotypes) == len(variant.genotypes)

def test_parse_cnvnator():
    """docstring for test_parse_cnvnator"""
    vcf_obj = VCF(one_cnvnator)
    for variant in vcf_obj:
        assert variant



from scout.parse.variant.gene import parse_genes
from scout.parse.variant.transcript import parse_transcripts


def test_get_genes(cyvcf2_variant):
    csq_entry = "C|missense_variant|MODERATE|POC1A|ENSG00000164087|"\
    "Transcript|ENST00000296484|protein_coding|4/11||ENST00000296484.2"\
    ":c.322A>G|ENSP00000296484.2:p.Ser108Gly|362|322|108|S/G|Agt/Ggt"\
    "|||-1|HGNC|24488||CCDS2846.1|ENSP00000296484|POC1A_HUMAN|"\
    "B2RDV4_HUMAN|UPI000045711C|deleterious|possibly_damaging|"\
    "PROSITE_profiles:PS50082&PROSITE_profiles:PS50294&hmmpanther"\
    ":PTHR22847:SF319&hmmpanther:PTHR22847&Gene3D:2.130.10.10&"\
    "Pfam_domain:PF00400&SMART_domains:SM00320&Superfamily_domains:"\
    "SSF50978|||||,C|missense_variant|MODERATE|POC1A|ENSG00000164087"\
    "|Transcript|ENST00000394970|protein_coding|4/10||"\
    "ENST00000394970.2:c.322A>G|ENSP00000378421.2:p.Ser108Gly|640"\
    "|322|108|S/G|Agt/Ggt|||-1|HGNC|24488||CCDS54592.1|ENSP00000378421"\
    "|POC1A_HUMAN||UPI00006633C6|deleterious|benign|"\
    "Superfamily_domains:SSF50978&SMART_domains:SM00320&Pfam_domain"\
    ":PF00400&Gene3D:2.130.10.10&hmmpanther:PTHR22847&hmmpanther:"\
    "PTHR22847:SF319&PROSITE_profiles:PS50082&PROSITE_profiles:PS50294"\
    "|||||,C|missense_variant|MODERATE|POC1A|ENSG00000164087|"\
    "Transcript|ENST00000474012|protein_coding|4/11||ENST00000474012"\
    ".1:c.208A>G|ENSP00000418968.1:p.Ser70Gly|425|208|70|S/G|Agt"\
    "/Ggt|||-1|HGNC|24488||CCDS54591.1|ENSP00000418968|POC1A_HUMAN|"\
    "B2RDV4_HUMAN|UPI0000E1FCF5|tolerated|possibly_damaging|Superfami"\
    "ly_domains:SSF50978&SMART_domains:SM00320&Gene3D:2.130.10.10&"\
    "Pfam_domain:PF00400&hmmpanther:PTHR22847:SF319&hmmpanther:"\
    "PTHR22847&PROSITE_profiles:PS50294&PROSITE_profiles:PS50082|||||"
    
    csq_header = "Allele|Consequence|IMPACT|SYMBOL|Gene|Feature_type|"\
    "Feature|BIOTYPE|EXON|INTRON|HGVSc|HGVSp|cDNA_position|CDS_position|"\
    "Protein_position|Amino_acids|Codons|Existing_variation|DISTANCE|"\
    "STRAND|SYMBOL_SOURCE|HGNC_ID|TSL|CCDS|ENSP|SWISSPROT|TREMBL|UNIPARC|"\
    "SIFT|PolyPhen|DOMAINS|HGVS_OFFSET|MOTIF_NAME|MOTIF_POS|HIGH_INF_POS|"\
    "MOTIF_SCORE_CHANGE"
    
    header = [word.upper() for word in csq_header.split('|')]
    
    ## GIVEN a variant with vep annotation and a vep header
    raw_transcripts = (
        dict(
            zip(
                header, 
                csq_entry.split('|')
                )
            )
        for transcript_info in csq_entry.split(',')
    )
    parsed_transcripts = []
    for parsed_transcript in parse_transcripts(raw_transcripts, allele='C'):
        parsed_transcripts.append(parsed_transcript)
    
    # variant = cyvcf2_variant
    # variant.INFO['CSQ'] = csq_entry
    genes = parse_genes(parsed_transcripts)

    for gene in genes:
        assert gene['hgnc_id'] == 24488
    assert len(genes) == 1



from pprint import pprint as pp

from scout.update.panel import update_panel
from scout.utils.date import get_date

def test_update_panel_version(panel_database, case_obj):
    adapter = panel_database
    adapter._add_case(case_obj)
    
    ## GIVEN an adapter with a case with gene panels
    case_obj = adapter.case_collection.find_one()
    panel_obj = adapter.panel_collection.find_one()
    
    case_id = case_obj['_id']
    
    # There is infirmation about a panel both in the panel collection
    # and on the case object. This is fine until one starts to manipulate the objects
    panel = case_obj['panels'][0]
    
    panel_version = panel['version']
    panel_name = panel['panel_name']
    panel_id = panel['panel_id']

    new_panel_version = panel_version + 1
    
    ## WHEN updating the panel version
    
    update_panel(adapter, panel_name, panel_version, new_panel_version)

    
    ## THEN assert that the panel version was updated for the panel object
    
    updated_panel_obj = adapter.panel_collection.find_one(
        {
            'panel_name': panel_name,
            'version': new_panel_version
        })
    
    
    assert updated_panel_obj['version'] == new_panel_version
    

    case_obj = adapter.case_collection.find_one({'_id': case_id})

    for panel in case_obj['panels']:
        assert panel['version'] == new_panel_version
    

def test_update_panel_date(panel_database, case_obj):
    adapter = panel_database
    adapter._add_case(case_obj)
    
    ## GIVEN an adapter with a case with gene panels
    new_date_obj = get_date('2015-03-12')
    
    case_obj = adapter.case_collection.find_one()
    panel_obj = adapter.panel_collection.find_one()
    
    case_id = case_obj['_id']
    
    # There is infirmation about a panel both in the panel collection
    # and on the case object. This is fine until one starts to manipulate the objects
    case_panel = case_obj['panels'][0]
    
    panel_version = case_panel['version']
    panel_name = case_panel['panel_name']
    panel_id = case_panel['panel_id']

    ## WHEN updating the panel version
    
    update_panel(adapter, panel_name, panel_version=None, new_date=new_date_obj)
    
    ## THEN assert that the panel version was updated both in panel and case
    
    updated_panel_obj = adapter.panel_collection.find_one(
        {
            'panel_name': panel_name,
            'version': panel_version
        })
    
    assert updated_panel_obj['date'] == new_date_obj
    
    case_obj = adapter.case_collection.find_one({'_id': case_id})
    
    for panel in case_obj['panels']:
        assert panel['updated_at'] == new_date_obj

def test_update_panel_version_multiple(panel_database, case_obj):
    adapter = panel_database
    adapter._add_case(case_obj)
    
    case_obj['_id'] = 'test_2'
    # Add another case with same panels
    adapter._add_case(case_obj)
    
    ## GIVEN an adapter with a case with gene panels
    case_obj = adapter.case_collection.find_one()
    panel_obj = adapter.panel_collection.find_one()
    
    # There is infirmation about a panel both in the panel collection
    # and on the case object. This is fine until one starts to manipulate the objects
    panel = case_obj['panels'][0]
    
    panel_version = panel['version']
    panel_name = panel['panel_name']
    panel_id = panel['panel_id']

    new_panel_version = panel_version + 1
    
    ## WHEN updating the panel version
    
    update_panel(adapter, panel_name, panel_version, new_panel_version)
    
    ## THEN assert that the panel version was updated both in panel and case
    
    updated_panel_obj = adapter.panel_collection.find_one(
        {
            'panel_name': panel_name,
            'version': new_panel_version
        })
    
    assert updated_panel_obj['version'] == new_panel_version
    
    for case_obj in adapter.case_collection.find(): 
    
        for panel in case_obj['panels']:
            assert panel['version'] == new_panel_version



from scout.parse.variant.conservation import (parse_conservation, parse_conservations)

def test_parse_conservation(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with some GERP information
    variant.INFO['dbNSFP_GERP___RS'] = 3.7
    ## WHEN parsing conservation
    ## THEN assert that the field is parsed correct
    assert parse_conservation(variant, 'dbNSFP_GERP___RS') == ['Conserved']

def test_parse_conservation_multiple_terms(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with multiple GERP annotations
    variant.INFO['dbNSFP_GERP___RS'] = 3.7, -0.34

    ## WHEN parsing conservation
    ## THEN assert that all terms are returned
    assert parse_conservation(variant, 'dbNSFP_GERP___RS') == ['Conserved', 'NotConserved']

def test_parse_conservations(cyvcf2_variant):
    variant = cyvcf2_variant
    ## GIVEN a variant with multiple conservation annotations
    variant.INFO['dbNSFP_GERP___RS'] = 4.6,0
    variant.INFO['dbNSFP_phastCons100way_vertebrate'] = 0.8
    variant.INFO['dbNSFP_phyloP100way_vertebrate'] = 2.4

    ## WHEN parsing conservation
    ## THEN assert that all terms are returned

    conservations = parse_conservations(variant)

    assert conservations['gerp'] == ['Conserved', 'NotConserved']
    assert conservations['phast'] == ['Conserved']
    assert conservations['phylop'] == ['NotConserved']



import logging

from scout.models.phenotype_term import HpoTerm

LOG = logging.getLogger(__name__)

def build_hpo_term(hpo_info):
    """Build a hpo_term object
    
    Check that the information is correct and add the correct hgnc ids to the 
    array of genes.
    
        Args:
            hpo_info(dict)
        
        Returns:
            hpo_obj(scout.models.HpoTerm): A dictionary with hpo information
    
    """
    
    try:
        hpo_id = hpo_info['hpo_id']
    except KeyError:
        raise KeyError("Hpo terms has to have a hpo_id")

    LOG.debug("Building hpo term %s", hpo_id)

    # Add description to HPO term
    try:
        description = hpo_info['description']
    except KeyError:
        raise KeyError("Hpo terms has to have a description")

    hpo_obj = HpoTerm(
        hpo_id = hpo_id,
        description = description
    )
    
    # Add links to hgnc genes if any
    hgnc_ids = hpo_info.get('genes', set())
    if hgnc_ids:
        hpo_obj['genes'] = list(hgnc_ids)
    
    return hpo_obj




# from scout.commands import cli
# from click.testing import CliRunner
# import scout.adapter
# import pytest
#
# def test_load_database(database_name):
#
#     runner = CliRunner()
#     result = runner.invoke(cli, [
#             '-db', database_name,
#             '--loglevel', 'DEBUG',
#             'load',
#             'institute',
#             '--internal_id', 'cust000',
#             '--display_name', 'test',
#         ])
#     assert result.exit_code == 0



import logging

from scout.constants import REV_CLINSIG_MAP

logger = logging.getLogger(__name__)


def parse_clnsig(acc, sig, revstat, transcripts):
    """Get the clnsig information

    Args:
        acc(str): The clnsig accession number, raw from vcf
        sig(str): The clnsig significance score, raw from vcf
        revstat(str): The clnsig revstat, raw from vcf
        transcripts(iterable(dict))

    Returns:
        clnsig_accsessions(list): A list with clnsig accessions
    """
    clnsig_accsessions = []
    
    if acc:
    # New format of clinvar allways have integers as accession numbers
        try:
            acc = int(acc)
        except ValueError:
            pass
        # There are sometimes different separators so we need to check which
        # one to use
        if isinstance(acc, int):
            revstat_groups = []
            if revstat:
                revstat_groups = [rev.lstrip('_') for rev in revstat.split(',')]
            
            sig_groups = []
            if sig:
                for significance in sig.split('/'):
                    splitted_word = significance.split('_')
                    sig_groups.append(' '.join(splitted_word[:2]))
            
            clnsig_accsessions.append({
                'value': ', '.join(sig_groups),
                'accession': int(acc),
                'revstat': ', '.join(revstat_groups),
            })
        else:
            # There are sometimes different separators so we need to check which
            # one to use
            acc_groups = acc.split('|')
            sig_groups = sig.split('|')
            revstat_groups = revstat.split('|')
            for acc_group, sig_group, revstat_group in zip(acc_groups, sig_groups, revstat_groups):
                accessions = acc_group.split(',')
                significances = sig_group.split(',')
                revstats = revstat_group.split(',')
                for accession, significance, revstat in zip(accessions, significances, revstats):
                    clnsig_accsessions.append({
                        'value': int(significance),
                        'accession': accession,
                        'revstat': revstat,
                    })

    elif transcripts:
        clnsig = set()
        for transcript in transcripts:
            for annotation in transcript.get('clinsig', []):
                clnsig.add(annotation)
        for annotation in clnsig:
            clnsig_accsessions.append({'value': annotation})

    return clnsig_accsessions



import logging
import click

LOG = logging.getLogger(__name__)


@click.command('whitelist', short_help='Display whitelist')
@click.pass_context
def whitelist(context):
    """Show all objects in the whitelist collection"""
    LOG.info("Running scout view users")
    adapter = context.obj['adapter']

    ## TODO add a User interface to the adapter
    for whitelist_obj in adapter.whitelist_collection.find():
        click.echo(whitelist_obj['_id'])




import logging

from pprint import pprint as pp

import click

LOG = logging.getLogger(__name__)


@click.command('institute', short_help='Update institute for a case')
@click.argument('institute-id')
@click.option('-c', '--coverage-cutoff',
    type=int,
    help="Set a new coverage cutoff for a institute", 
)
@click.option('-f', '--frequency-cutoff',
    type=float,
    help="Set a new frequency cutoff for a institute", 
)
@click.option('-s', '--sanger-recipient',
    help="Specify email adress for a existing user that should be added to the institute",
)
@click.option('-d', '--display-name',
    help="Set a new display name for a insitute",
)
@click.option('-r', '--remove-sanger',
    help="Specify email adress for a existing user that should be removed from sanger recipients",
)
@click.pass_context
def institute(context, institute_id, sanger_recipient, coverage_cutoff, frequency_cutoff, 
              display_name, remove_sanger):
    """
    Update an institute
    """
    adapter = context.obj['adapter']
    LOG.info("Running scout update institute")
    
    try:
        adapter.update_institute(
            internal_id=institute_id, 
            sanger_recipient=sanger_recipient, 
            coverage_cutoff=coverage_cutoff, 
            frequency_cutoff=frequency_cutoff, 
            display_name=display_name,
            remove_sanger=remove_sanger,
            )
    except Exception as err:
        LOG.warning(err)
        context.abort()



"""
Cli functions to setup scout

There are two options. 
`scout setup demo` will setup a database that are loaded with more example 
data but the gene definitions etc are reduced.

`scout setup database` will create a full scale instance of scout. There will not be any cases 
and one admin user is added.



"""
import logging
import datetime

import pymongo
import click

from pprint import pprint as pp

# Adapter stuff
from scout.adapter.mongo import MongoAdapter
from scout.adapter.client import get_connection
from pymongo.errors import (ConnectionFailure, ServerSelectionTimeoutError)

from scout.load.setup import setup_scout

LOG = logging.getLogger(__name__)

def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()

@click.command('database', short_help='Setup a basic scout instance')
@click.option('-i', '--institute-name', type=str)
@click.option('-u', '--user-name', type=str)
@click.option('-m', '--user-mail', type=str)
@click.option('--api-key', help='Specify the api key')
@click.option('--yes', 
    is_flag=True, 
    callback=abort_if_false,
    expose_value=False,
    prompt='This will delete existing database, do you wish to continue?')
@click.pass_context
def database(context, institute_name, user_name, user_mail, api_key):
    """Setup a scout database."""
    LOG.info("Running scout setup database")

    # Fetch the omim information
    api_key = api_key or context.obj.get('omim_api_key')
    if not api_key:
        LOG.warning("Please provide a omim api key with --api-key")
        context.abort()

    institute_name = institute_name or context.obj['institute_name']
    user_name = user_name or context.obj['user_name']
    user_mail = user_mail or context.obj['user_mail']

    adapter = context.obj['adapter']

    LOG.info("Setting up database %s", context.obj['mongodb'])
    
    setup_scout(
        adapter=adapter,
        institute_id=institute_name, 
        user_name=user_name, 
        user_mail = user_mail, 
        api_key=api_key
    )

@click.command('demo', short_help='Setup a scout demo instance')
@click.pass_context
def demo(context):
    """Setup a scout demo instance. This instance will be populated with a
       case, a gene panel and some variants.
    """
    LOG.info("Running scout setup demo")
    institute_name = context.obj['institute_name']
    user_name = context.obj['user_name']
    user_mail = context.obj['user_mail']

    adapter = context.obj['adapter']

    LOG.info("Setting up database %s", context.obj['mongodb'])
    
    setup_scout(
        adapter=adapter,
        institute_id=institute_name, 
        user_name=user_name, 
        user_mail = user_mail, 
        demo=True
    )

from scout.demo.resources.generate_test_data import (generate_hgnc, generate_genemap2, generate_mim2genes, 
generate_exac_genes, generate_ensembl_genes, generate_ensembl_transcripts, generate_hpo_files)
from scout.demo import panel_path

from scout.parse.panel import parse_gene_panel

@click.group()
@click.option('-i', '--institute',
    default='cust000',
    show_default=True,
    help='Name of initial institute',
)
@click.option('-e', '--user-mail',
    default='clark.kent@mail.com',
    show_default=True,
    help='Mail of initial user',
)
@click.option('-n', '--user-name',
    default='Clark Kent',
    show_default=True,
    help='Name of initial user',
)
@click.pass_context
def setup(context, institute, user_mail, user_name):
    """
    Setup scout instances.
    """

    context.obj['institute_name'] = institute
    context.obj['user_name'] = user_name
    context.obj['user_mail'] = user_mail

    if context.invoked_subcommand == 'demo':
        # Update context.obj settings here
        LOG.debug("Change database name to scout-demo")
        context.obj['mongodb'] = 'scout-demo'

    LOG.info("Setting database name to %s", context.obj['mongodb'])
    LOG.debug("Setting host to %s", context.obj['host'])
    LOG.debug("Setting port to %s", context.obj['port'])
    try:
        client = get_connection(
                    host=context.obj['host'],
                    port=context.obj['port'],
                    username=context.obj['username'],
                    password=context.obj['password'],
                    mongodb = context.obj['mongodb']
                )
    except ConnectionFailure:
        context.abort()

    LOG.info("connecting to database %s", context.obj['mongodb'])
    database = client[context.obj['mongodb']]
    LOG.info("Test if mongod is running")
    try:
        LOG.info("Test if mongod is running")
        database.test.find_one()
    except ServerSelectionTimeoutError as err:
        LOG.warning("Connection could not be established")
        LOG.warning("Please check if mongod is running")
        context.abort()

    LOG.info("Setting up a mongo adapter")
    mongo_adapter = MongoAdapter(database)
    context.obj['adapter'] = mongo_adapter


setup.add_command(database)
setup.add_command(demo)



# -*- coding: utf-8 -*-
from functools import partial
import logging

import click

LOG = logging.getLogger(__name__)

@click.command(short_help='Upload research variants')
@click.option('-c', '--case-id', help='family or case id')
@click.option('-i', '--institute', help='institute id of related cases')
@click.option('-f', '--force', is_flag=True, help='upload without request')
@click.pass_context
def research(context, case_id, institute, force):
    """Upload research variants to cases

        If a case is specified, all variants found for that case will be
        uploaded.

        If no cases are specified then all cases that have 'research_requested'
        will have there research variants uploaded
    """
    LOG.info("Running scout load research")
    adapter = context.obj['adapter']

    if case_id:
        if not institute:
            # There was an old way to create case ids so we need a special case to handle this
            # Assume institute-case combo
            splitted_case = case_id.split('-')
            # Check if first part is institute, then we know it is the old format
            if len(splitted_case) > 1:
                institute_obj = adapter.institute(splitted_case[0])
                if institute_obj:
                    institute = institute_obj['_id']
                    case_id = splitted_case[1]
        case_obj = adapter.case(institute_id=institute, case_id=case_id)
        if case_obj is None:
            LOG.warning("No matching case found")
            context.abort()
        else:
            case_objs = [case_obj]
    else:
        # Fetch all cases that have requested research
        case_objs = adapter.cases(research_requested=True)

    default_threshold = 8
    files = False
    for case_obj in case_objs:
        if force or case_obj['research_requested']:
            # Test to upload research snvs
            if case_obj['vcf_files'].get('vcf_snv_research'):
                files = True
                adapter.delete_variants(case_id=case_obj['_id'],
                                        variant_type='research',
                                        category='snv')

                LOG.info("Load research SNV for: %s", case_obj['_id'])
                adapter.load_variants(
                    case_obj=case_obj,
                    variant_type='research',
                    category='snv',
                    rank_threshold=default_threshold,
                    )

            # Test to upload research svs
            if case_obj['vcf_files'].get('vcf_sv_research'):
                files = True
                adapter.delete_variants(case_id=case_obj['_id'],
                                        variant_type='research',
                                        category='sv')
                LOG.info("Load research SV for: %s", case_obj['_id'])
                adapter.load_variants(
                    case_obj=case_obj,
                    variant_type='research',
                    category='sv',
                    rank_threshold=default_threshold,
                    )

            # Test to upload research cancer variants
            if case_obj['vcf_files'].get('vcf_cancer_research'):
                files = True
                adapter.delete_variants(case_id=case_obj['_id'],
                                        variant_type='research',
                                        category='cancer')

                LOG.info("Load research cancer for: %s", case_obj['_id'])
                adapter.load_variants(
                    case_obj=case_obj,
                    variant_type='research',
                    category='cancer',
                    rank_threshold=default_threshold,
                    )
            if not files:
                LOG.warning("No research files found for case %s", case_id)
                context.abort()
            case_obj['is_research'] = True
            case_obj['research_requested'] = False
            adapter.update_case(case_obj)
        else:
            LOG.warn("research not requested, use '--force'")



# -*- coding: utf-8 -*-
import re
from collections import OrderedDict

from intervaltree import (IntervalTree, Interval)

from scout.parse.cytoband import parse_cytoband
from scout.resources import cytobands_path
from scout.utils.handle import get_file_handle

from .indexes import INDEXES

from .acmg import (ACMG_COMPLETE_MAP, ACMG_OPTIONS, ACMG_CRITERIA, ACMG_MAP, REV_ACMG_MAP)
from .so_terms import (SO_TERMS, SO_TERM_KEYS, SEVERE_SO_TERMS)
from .variant_tags import (CONSEQUENCE, CONSERVATION, DISMISS_VARIANT_OPTIONS,
                           FEATURE_TYPES, SPIDEX_LEVELS, SPIDEX_HUMAN, SV_TYPES,
                           GENETIC_MODELS, VARIANT_CALL, MANUAL_RANK_OPTIONS, MOSAICISM_OPTIONS)
from .case_tags import (ANALYSIS_TYPES, SEX_MAP, REV_SEX_MAP, PHENOTYPE_MAP,
                        REV_PHENOTYPE_MAP, CASE_STATUSES, VERBS_MAP, VERBS)
from .clnsig import (CLINSIG_MAP, REV_CLINSIG_MAP)
from .phenotype import (PHENOTYPE_GROUPS, COHORT_TAGS)
from .file_types import FILE_TYPE_MAP
from .clinvar import (CLINVAR_HEADER, CASEDATA_HEADER)
from .variants_export import (EXPORT_HEADER, VCF_HEADER, MT_EXPORT_HEADER, VERIFIED_VARIANTS_HEADER)

cytobands_handle = get_file_handle(cytobands_path)

COLLECTIONS = [
    'hgnc_gene',
    'user',
    'whitelist',
    'institute',
    'event',
    'case',
    'gene_panel',
    'hpo_term',
    'disease_term',
    'variant',
    'acmg',
]

BUILDS = ['37', '38']

CYTOBANDS = parse_cytoband(cytobands_handle)

CHROMOSOMES = ('1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12',
               '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', 'X',
               'Y', 'MT')

# Maps chromosomes to integers
CHROMOSOME_INTEGERS = {chrom: i+1 for i,chrom in enumerate(CHROMOSOMES)}


PAR_COORDINATES = {
    '37': {
        'X': IntervalTree([Interval(60001, 2699521, 'par1'),
                           Interval(154931044, 155260561, 'par2')]),
        'Y': IntervalTree([Interval(10001, 2649521, 'par1'),
                           Interval(59034050, 59363567, 'par2')])
    },
    '38': {
        'X': IntervalTree([Interval(10001, 2781480, 'par1'),
                           Interval(155701383, 156030896, 'par2')]),
        'Y': IntervalTree([Interval(10001, 2781480, 'par1'),
                           Interval(56887903, 57217416, 'par2')])
    },
}

CALLERS = {
    'snv': [{
            'id': 'gatk',
            'name': 'GATK',
        }, {
            'id': 'freebayes',
            'name': 'Freebayes',
        }, {
            'id': 'samtools',
            'name': 'SAMtools',
        },
        {
            'id': 'bcftools',
            'name': 'Bcftools',
        }
    ],
    'cancer': [{
            'id': 'mutect',
            'name': 'MuTect',
        }, {
            'id': 'pindel',
            'name': 'Pindel',
        }, {
            'id': 'gatk',
            'name': 'GATK',
        }, {
            'id': 'freebayes',
            'name': 'Freebayes',
    }],
    'sv': [{
            'id': 'cnvnator',
            'name': 'CNVnator',
        }, {
            'id': 'delly',
            'name': 'Delly',
        }, {
            'id': 'tiddit',
            'name': 'TIDDIT',
        }, {
            'id': 'manta',
            'name': 'Manta',
    }],
    'str': [{
           'id': 'expansionhunter',
           'name': 'ExpansionHunter'
   }]
}

BND_ALT_PATTERN = re.compile(r".*[\],\[](.*?):(.*?)[\],\[]")
CHR_PATTERN = re.compile(r'(chr)?(.*)', re.IGNORECASE)



# from scout.commands import cli
# from click.testing import CliRunner

vcf_file = "tests/fixtures/337334.clinical.vcf"
ped_file = "tests/fixtures/337334.ped"
scout_config = "tests/fixtures/scout_config_test.ini"

# def test_load_database(database_setup):
#     """docstring for test_load_database"""
#     runner = CliRunner()
#     args = open(database_setup, 'r')
#     print(args.read())
#     result = runner.invoke(cli, [
#             '-c', database_setup,
#             'load',
#             '--vcf_file', vcf_file,
#             '--ped_file', ped_file,
#             '--family_type', 'ped',
#             '--owner', 'aowner',
#         ])
#     assert result.exit_code == 0
#
# def test_load_database_config(database_setup):
#     """docstring for test_load_database"""
#     runner = CliRunner()
#     args = open(database_setup, 'r')
#     result = runner.invoke(cli, [
#             '-c', database_setup,
#             'load',
#             '--scout_config_file', scout_config,
#         ])
#     assert result.exit_code == 0
#
#
# def test_load_database_no_vcf(database_setup, variant_file):
#     """docstring for test_load_database"""
#     runner = CliRunner()
#     args = open(database_setup, 'r')
#     result = runner.invoke(cli, [
#             '-c', database_setup,
#             'load',
#         ])
#     assert result.exit_code == 1
#
# def test_load_database_no_ped(database_setup, variant_file):
#     """docstring for test_load_database"""
#     runner = CliRunner()
#     args = open(database_setup, 'r')
#     result = runner.invoke(cli, [
#             '-c', database_setup,
#             'load',
#             '--vcf_file', vcf_file,
#         ])
#     assert result.exit_code == 1
#
# def test_load_database_no_owner(database_setup):
#     """docstring for test_load_database"""
#     runner = CliRunner()
#     args = open(database_setup, 'r')
#     print(args.read())
#     result = runner.invoke(cli, [
#             '-c', database_setup,
#             'load',
#             '--vcf_file', vcf_file,
#             '--ped_file', ped_file,
#             '--family_type', 'ped',
#         ])
#     assert result.exit_code == 1



import logging
import pymongo
from scout.parse.clinvar import clinvar_submission_header,clinvar_submission_lines

LOG = logging.getLogger(__name__)

def get_test_submission_variant():
    """Returns a test clinvar variant submission object"""

    variant_subm_obj = {
        '_id' : "case1_a99ab86f2cb3bc18b993d740303ba27f",
        'csv_type' : "variant",
        'case_id' : "case1",
        'category' : "snv",
        'local_id' : "a99ab86f2cb3bc18b993d740303ba27f",
        'linking_id' : "a99ab86f2cb3bc18b993d740303ba27f",
        'chromosome' : "5",
        'start' : "7666888",
        'stop' : "7666888",
        'ref' : "A",
        'alt' : "T",
        'clinsig' : "Pathogenic"
    }
    return variant_subm_obj


def get_test_submission_case():
    """Returns a test casedata submission object"""

    casedata_subm_obj = {
        '_id' : "case1_a99ab86f2cb3bc18b993d740303ba27f_subj1",
        'csv_type': "casedata",
        'case_id' : "case1",
        'linking_id' : "a99ab86f2cb3bc18b993d740303ba27f",
        'individual_id' : "subj1",
        'clin_features' : "HP:0001392"
    }
    return casedata_subm_obj


def get_new_submission(adapter, user_obj, institute_obj):

    # Given a valid user id
    user_id = user_obj['_id']
    assert user_id

    # And a valid institute id
    institute_id = institute_obj['_id']
    assert institute_id

    # Check that a new clinvar submission can be created
    new_submission_id = adapter.create_submission( user_id=user_id, institute_id=institute_id)
    return new_submission_id


def test_create_submission(adapter, user_obj, institute_obj):
    """Test create a clinvar submission"""

    # Assert that a new submission can be created
    submission_id = get_new_submission(adapter, user_obj, institute_obj)
    assert submission_id

    # Check that submission is returned when collecting submissions by user_id and institute_id
    submissions = adapter.clinvar_submissions(user_obj['_id'], institute_obj['_id'])
    assert submissions[0]['_id'] == submission_id


def test_delete_submission(adapter, user_obj, institute_obj):
    """Test delete a clinvar submission providing its ID"""
    # Get the ID of an existing submission
    submission_id = get_new_submission(adapter, user_obj, institute_obj)

    # Check that it is possible to delete the submission
    deleted_objects = adapter.delete_submission(submission_id)
    assert deleted_objects == (0,1) # deleted 1 submission with 0 objects inside


def test_clinvar_submission_status(adapter, user_obj, institute_obj):
    """Test the function that returns an open clinvar submission object"""

    user_id = user_obj['_id']
    institute_id = institute_obj['_id']

    # Check that a new clinvar submission can be created
    submission_obj = adapter.get_open_clinvar_submission( user_id=user_id, institute_id=institute_id)
    assert submission_obj

    # And that its 'status' is set to "open"
    assert submission_obj['status'] == "open"

    # Update submission status to 'closed'
    updated_submission = adapter.update_clinvar_submission_status(user_id , submission_obj['_id'], "closed")

    # And that now its 'status' is set to "closed"
    assert updated_submission['status'] == "closed"


def test_update_clinvar_id(adapter, user_obj, institute_obj):
    """record an official clinvar submission name for a submission"""

    submission_id = get_new_submission(adapter, user_obj, institute_obj)

    # Update the submission with the official clinvar name
    updated_submission = adapter.update_clinvar_id(clinvar_id='SUB0001', submission_id=submission_id)

    # Assert that the submission was updated
    assert adapter.get_clinvar_id(submission_id) == "SUB0001"


def test_add_remove_subm_objects(adapter, user_obj, institute_obj):
    """Test adding variant and casedata objects to a submission"""

    # Get the ID of an existing submission
    submission_obj = adapter.get_open_clinvar_submission( user_id=user_obj['_id'], institute_id=institute_obj['_id'])

    # Given a test variant and its associated casedata
    variant_data = [ get_test_submission_variant() ] # a list of 1 dictionary element
    case_data = [ get_test_submission_case() ] # a list of 1 dictionary element
    subm_objs = (variant_data, case_data)

    # assert that clinvar collection in database is empty
    submission_objects = list(adapter.clinvar_collection.find())
    assert len(submission_objects) == 0

    # Add objects to submission
    updated_subm = adapter.add_to_submission( submission_obj['_id'], subm_objs)

    # Assert that 'variant_data' value of submission is updated
    assert updated_subm['variant_data'] == [variant_data[0].get('_id')]

    # Assert that 'case_data' value of submission is updated
    assert updated_subm['case_data'] == [case_data[0].get('_id')]

    # assert that clinvar collection in database has now 2 elements
    submission_objects = list(adapter.clinvar_collection.find())
    assert len(submission_objects) == 2

    # check if it is possible to create CSV files header for variant and casedata objects
    variants_file_header = clinvar_submission_header(submission_objects, 'variant_data')
    assert len(variants_file_header.keys()) > 0
    casedata_file_header = clinvar_submission_header(submission_objects, 'case_data')
    assert len(casedata_file_header.keys()) > 0

    # check if it is possible to create CSV files lines for variant and casedata objects
    variants_file_lines = clinvar_submission_lines(submission_objects, variants_file_header)
    assert len(variants_file_lines[0].split(',')) == len(variants_file_header.keys()) # variant file header and line have the same number of columns
    casedata_file_lines = clinvar_submission_lines(submission_objects, casedata_file_header)
    assert len(casedata_file_lines[0].split(',')) == len(casedata_file_header.keys()) # casedata file header and line have the same number of columns

    # assert that one of these objects is the variant object
    assert variant_data == adapter.clinvar_objs(submission_obj['_id'], 'variant_data')

    # assert that one of these objects is the case data object
    assert case_data == adapter.clinvar_objs(submission_obj['_id'], 'case_data')

    # assert that a variant is present for case 'case1'
    assert adapter.case_to_clinVars('case1') == {variant_data[0]['local_id'] : variant_data[0]}

    # Removal of clinvar objects from submission and from clinvar collection in database
    # remove case_data object
    updated_submission = adapter.delete_clinvar_object(object_id=case_data[0]['_id'], object_type='case_data', submission_id=submission_obj['_id'])
    assert updated_submission['case_data'] == []

    # remove variant object
    updated_submission = adapter.delete_clinvar_object(object_id=variant_data[0]['_id'], object_type='variant_data', submission_id=submission_obj['_id'])
    assert updated_submission['variant_data'] == []

    # assert that there are no objects left in clinvar collection
    submission_objects = list(adapter.clinvar_collection.find())
    assert len(submission_objects) == 0
